#!/usr/bin/env jython
# .+
# .context    : Application View Controller
# .title      : A random generation of count down windows (Swing),
#		GUI programmatically generated
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Torino - Italy
# .creation   :	22-Oct-2009
# .copyright  : (c) 2009 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-

from javax import swing			# swing toolkit bindings
from java import awt			# awt toolkit bindings

from avc import *			# AVC for Swing

from random import randint		# random integer generator
import sys				# system support

FIRST_COUNT_DELAY = 1000		# let avc_init be called
COUNTDOWN_PERIOD = 500			# count down at 2 unit per second
MAX_CREATION_PERIOD = 4000		# create a new count down at 1/2 this


class Countdown(AVC):
  """
  A countdown counter displayed in a Label widget. Count starts at given
  value. When count reaches zero the counter and its GUI are destroyed.
  """

  def __init__(self,count_start=10):

    # create GUI
    self.root = swing.JFrame('AVC Swing countdown example',size=(350,60),
      defaultCloseOperation = swing.JFrame.EXIT_ON_CLOSE)
    self.root.layout = awt.FlowLayout()
    self.root.add(swing.JLabel('counter',name='counter__label',))
    self.root.show()

    # init the counter variable 
    self.counter = count_start

    # connect counter variable with label widget
    self.avc_connect(self.root)

    # start count down
    self.timer = swing.Timer(COUNTDOWN_PERIOD,None)
    self.timer.actionPerformed = self.decrementer
    self.timer.start()
  

  def decrementer(self,*args):
    "Counter decrementer."
    self.counter -= 1

    if not self.counter:
      # counter reached zero: destroy this countdown and its GUI
      self.root.dispose()


class Example(AVC):
  """
  Continuously create at random intervals windows with a countdown from 10 to 0.
  When a countdown reaches zero, its window is destroyed. Also create a main
  window with a "close all" button.
  """

  def __init__(self):

    # create main window
    self.root = swing.JFrame('AVC Swing countdown example',size=(350,60),
      defaultCloseOperation = swing.JFrame.EXIT_ON_CLOSE)
    self.root.layout = awt.FlowLayout()
    self.root.add(swing.JButton('CLOSE ALL',name='close_all__button',))
    self.root.show()

    # create the first countdown after avc_init call
    self.timer = swing.Timer(FIRST_COUNT_DELAY,None)
    self.timer.actionPerformed = self.new_countdown
    self.timer.start()

    # close all button connected variable
    self.close_all = False


  def new_countdown(self,event,count_start=10):
    "Create a new countdown"

    # create a new countdown
    Countdown(count_start)

    # autocall after a random delay
    self.timer.setDelay(MAX_CREATION_PERIOD)


  def close_all_changed(self,value):
    "Terminate program at 'close all' button pressing"
    for frame in self.root.getFrames():
       frame.dispose()
    sys.exit()


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables

#### END



# .+
# .context    : Application View Controller
# .title      : AVC package dummy init
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	7-Nov-2006
# .copyright  :	(c) 2006-2008 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with AVC.  If not, see <http://www.gnu.org/licenses/>.
#
# .-

# import core module
import avccore

# expose user interface
AVC = avccore.AVC
__all__ = ['AVC']

del avccore



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : A counter with count speed control (Qt4)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	7-Dec-2006
# .copyright  : (c) 2006 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


from PyQt4.QtCore import *		# Qt core
from PyQt4.QtGui import *		# Qt GUI interface
from PyQt4.uic import *			# ui files realizer
import sys				# system support

from avc import *			# AVC

UI_FILE = 'qt4_counter.ui'		# qt ui descriptor
LOW_SPEED = 0.5				#--
HIGH_SPEED = 0.1			#- low and high speed count period (sec)


class ExampleGUI(QApplication):
  "Counter GUI creation"

  def __init__(self):

    # create GUI
    QApplication.__init__(self,sys.argv)
    self.root = loadUi(UI_FILE)
    self.root.show()
    
  def timer_start(self,period,function):
    "Start a Qt timer calling back 'function' every 'period' seconds."
    self.timer1 = QTimer()
    QObject.connect(self.timer1,SIGNAL("timeout()"),function)
    self.timer1.start(int(period * 1000.0))

  def timer_set_period(self,period):
    "Set a new period to timer"
    self.timer1.stop()
    self.timer1.start(int(period * 1000.0))


class ExampleMain(AVC):
  """
  A counter displayed in a Label widget whose count speed can be
  accelerated by checking a check box.
  """

  def __init__(self,gui):

    # save GUI
    self.gui = gui

    # the counter variable and its speed status
    self.counter = 0
    self.high_speed = False

    # start incrementer timer
    self.gui.timer_start(LOW_SPEED,self.incrementer)


  def incrementer(self):
    """
    Counter incrementer: increment period = LOW_SPEED, if high speed
    is False, increment period = HIGH_SPEED otherwise.
    """
    self.counter += 1
    if self.high_speed:
      period = HIGH_SPEED
    else:
      period = LOW_SPEED
    self.gui.timer_set_period(period)

  def high_speed_changed(self,value):
    "Notify change of counting speed to terminal"
    if value:
      print 'counting speed changed to high'
    else:
      print 'counting speed changed to low'


#### MAIN

example_gui = ExampleGUI()		# create the application GUI
example = ExampleMain(example_gui)	# instantiate the application
example.avc_init()			# connect widgets with variables
example_gui.exec_()			# run Qt event loop until quit

#### END



#!/usr/bin/python
# -*- coding: utf-8 -*-
# .+
# .context    : Application View Controller
# .title      : AVC core
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	3-Nov-2006
# .copyright  :	(c) 2006-2015 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with AVC.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


# import required modules
import copy                            # object deep copy
import sys                             # command line option reading


class Error(Exception):
    """A generic error exception"""

    def __init__(self, value):
        Exception.__init__(self)
        self.value = value

    def __str__(self):
        return repr(self.value)


__author__ = 'Fabrizio Pollastri <f.pollastri@inrim.it>'
__license__ = '>= GPL v3'
__version__ = '0.9.0'


## load proper AVC widget toolkit binding according with the widget toolkit
# imported by application program

# supported toolkit names indexed by python binding module names
TOOLKITS = {
    'gtk': 'GTK+',
    'PyQt4': 'Qt4',
    'javax': 'Swing',
    'Tkinter': 'Tkinter',
    'wx': 'wxWidgets',
    }

# avc toolkit bindings indexed by python binding module names
AVC_BINDINGS = {
    'gtk': 'avcgtk',
    'PyQt4': 'avcqt4',
    'javax': 'avcswing',
    'Tkinter': 'avctk',
    'wx': 'avcwx',
    }

AVC_PREFIX = 'avc.'

# test which widget toolkit binding module is imported: if none raise an error.
for toolkit in TOOLKITS.keys():
    if sys.modules.has_key(toolkit):
        break
else:
    raise Error, \
        'No supported toolkit found: import it before AVC import.'

# found a supported toolkit: import its AVC binding
REAL = __import__(AVC_PREFIX + AVC_BINDINGS[toolkit], globals(),
                  locals(), [AVC_BINDINGS[toolkit]])

# command line option switches
OPT_VERBOSITY = '--avc-verbosity'

# separator between widget name part 1 and 2
WIDGET_NAME_SEP = '__'


class AVCcd:
    """AVC common data"""
    def __init__(self):
        self.toolkit_version = ''
        self.verbosity = 0
        self.view_period = 0.1
        self.widget_map = {}
        self.cogets = {}
        self.connections = {}
        self.connections_updates = {}
        self.connected_widgets = {}
        self.timer = None
AVCCD = AVCcd()


class AVC(object):
    """AVC programming interface"""

    def avc_init(self, verbosity=0, view_period=0.1):
        """Init AVC core logic"""

        # save parameters as globals
        AVCCD.verbosity = verbosity
        AVCCD.view_period = view_period

        # if any, get options from command line and override init arguments
        try:
            opt_switch_index = sys.argv.index(OPT_VERBOSITY)
            AVCCD.verbosity = int(sys.argv[opt_switch_index + 1])
        except:
            pass

        # do init specific to widget toolkit
        REAL.init(globals())

        # if verbosity > 0 , print header
        if AVCCD.verbosity > 0:
            print 'AVC ' + __version__ + ' - Activity Report'
            print 'widget toolkit binding: ' + TOOLKITS[toolkit] + ' v'\
                 + AVCCD.toolkit_version
            print 'program: ' + sys.argv[0]
            print 'verbosity: ' + str(AVCCD.verbosity)
            if AVCCD.view_period:
                print 'connection update mode: periodic, period='\
                     + str(AVCCD.view_period) + ' sec'
            else:
                print 'connection update mode: immediate'

        # connect widgets-variables in __main__ namespace
        self.avc_connect(REAL.toplevel_widgets())

        # if a sampled (periodic) update of all controls views is required,
        # start a periodic call to view update function.
        if AVCCD.view_period != 0.0:
            AVCCD.timer = REAL.timer(self.view_update,AVCCD.view_period)

    def avc_connect(self, toplevel):
        """
        For each widget at or below toplevel, having a matching name with
        an attribute of object_ object, create a widget-attribute connection.
        """

        # check for avc_init execution
        if not AVCCD.widget_map:
            raise Error,'avc_connect must be called after avc_init call.'

        # force top level widgets to be a list
        if toplevel.__class__ != list:
            toplevel = [toplevel]

        if AVCCD.verbosity > 3:
            print 'widget tree scansion from top level ' + str(toplevel)

        # for each widget in GUI ... 
        for widget, widget_name in self.get_widget(toplevel):

            # if widget is not supported: go to next widget
            if not AVCCD.widget_map.has_key(widget.__class__):

                if AVCCD.verbosity > 3:
                    print '  skip unsupported widget '\
                         + widget.__class__.__name__ + ',"'\
                         + widget_name + '"'

                continue

            # if the widget is already connected: go to next widget. 
            if AVCCD.connected_widgets.has_key(widget):

                if AVCCD.verbosity > 3:
                    print '  skip already connected widget '\
                         + widget.__class__.__name__ + ',"'\
                         + widget_name + '"'

                continue

            # control name is the widget name part before WIDGET_NAME_SEP
            # string, if present, otherwise is the whole widget name.
            control_name = widget_name.split(WIDGET_NAME_SEP)[0]

            # if no object attribute with the same name: go to next widget. 
            if not hasattr(self, control_name):

                if AVCCD.verbosity > 3:
                    print '  skip unmatched widget '\
                         + widget.__class__.__name__ + ',"'\
                         + widget_name + '"'

                continue

            ## there exists an application attribute with the same name

            # if the connection exists, get it from connections dictionary,
            # if the connection does not exists, create it.
            connection = AVCCD.connections.setdefault((control_name,
                    self), Connection(getattr(self, control_name)))

            # add widget to connection and mark widget as connected
            connection.add_widget(control_name, self, widget,widget_name)
            AVCCD.connected_widgets[widget] = connection


    def get_widget(self, widgets):
        """
        Widget tree iterator. Start from toplevel widgets and traverse their
        widgets trees in breath first mode returning for each widget its
        pointer and name.
        """

        # for each toplevel widget ...
        while widgets:
            children = []
            # for each widget in this level ...
            for widget in widgets:
                # return pointer and name of widget
                yield (widget, REAL.widget_name(widget))
                children += REAL.widget_children(widget)
                # children of this level are widgets of next level
                widgets = children


    def view_update(self):
        """Periodically update views for all scheduled cogets"""

        for connection in AVCCD.connections_updates.keys():
            setter = AVCCD.connections_updates[connection]
            # set the new control value in all widgets binded to this control
            # excluding the setting widget, if setter is a widget.
            for wal_widget in connection.wal_widgets:
                if wal_widget != setter:
                    try:
                        wal_widget.write(connection.control_value)
                    # on writing error terminate
                    except:
                        print 'error writing ' + str(connection.control_value) \
                             +' to '+ str(wal_widget) + ':\n',sys.exc_info()[1]
                        sys.exit()

        # clear all update requests
        AVCCD.connections_updates = {}


class Connection:
    """Widgets-variable connection"""


    def __init__(self, control_value=None):

        # set storage for control value, type, connected wal widget list,
        # value changed handler and coget
        self.control_value = control_value
        self.control_type = control_value.__class__
        self.wal_widgets = []
        self.set_handler = None
        self.object_ = None
        self.coget = None


    def add_widget(self,control_name,object_,widget,widget_name,):
        """ Add one widget to current connection """

        if not self.wal_widgets:

            self.object_ = object_

            if hasattr(object_, control_name + '_changed'):
                self.set_handler = getattr(object_, control_name
                         + '_changed')

            if AVCCD.cogets.has_key((control_name, object_.__class__)):
                self.coget = AVCCD.cogets[(control_name,
                        object_.__class__)]
            else:
                self.coget = AVCCD.cogets[(control_name,
                        object_.__class__)] = Coget(control_name)

                setattr(object_.__class__, control_name, self.coget)

            self.coget.add_connection(self)

            if AVCCD.verbosity > 0:
                print '  creating connection "' + control_name + '" in '\
                     + str(object_)
                print '    type: ' + str(self.control_type)
                print '    initial value: ' + str(self.control_value)
                if self.set_handler:
                    print '    connected handler ' + '"' + control_name\
                         + '_changed"'

        wal_widget = AVCCD.widget_map[widget.__class__]

        if AVCCD.verbosity > 1:
            print '  add widget ' + widget.__class__.__name__ + ',"'\
                 + widget_name + '" to connection "' + control_name\
                 + '"'

        self.wal_widgets.append(wal_widget(self, widget))

        self.wal_widgets[-1].write(self.control_value)

    def remove_widget(self, wal_widget):
        """
    Remove one widget from current connection. If it is the last
    one in the connection, delete the connection.
    """

        self.wal_widgets.remove(wal_widget)

        if AVCCD.verbosity > 1:
            print 'removing widget '\
                 + wal_widget.widget.__class__.__name__\
                 + ' from connection "' + self.coget.control_name\
                 + '" of ' + str(self.object_)

        del wal_widget.connection
        del wal_widget.widget

        if not self.wal_widgets:

            if AVCCD.verbosity > 0:
                print 'removing connection "' + self.coget.control_name\
                     + '" from ' + str(self.object_)

            del AVCCD.connections[(self.coget.control_name,
                                  self.object_)]

            self.coget.remove_connection(self)

            if AVCCD.connections_updates.has_key(self):
                del AVCCD.connections_updates[self]

            del self.control_value
            del self.control_type
            del self.wal_widgets
            del self.set_handler
            del self.object_
            del self.coget


class Coget(object):

    """A control object as data descriptor"""

    def __init__(self, control_name):
        """Create the coget control and bind it to one attribute in object"""

        self.control_name = control_name
        self.connections = []

    def add_connection(self, connection):
        """Add a connection"""

        self.connections.append(connection)

    def remove_connection(self, connection):
        """Remove a connection. If it is the last one, delete coget."""

        self.connections.remove(connection)
        if not self.connections:
            del AVCCD.cogets[(self.control_name,
                             connection.object_.__class__)]
            del self.control_name
            del self.connections

    def __get__(self, object_, classinfo):
        """Get control value"""

        return AVCCD.connections[(self.control_name,
                                 object_)].control_value

    def __set__(
        self,
        object_,
        value,
        setter=None,
        connection=None,
        ):
        """
    Set a new control value into application control variable. If setter
    is a widget (setter != None), call the application set handler, if exists.
    Update control view in all widgets binded to the control, if setter is
    a widget, do not update it.
    """

        if not connection:

            if AVCCD.connections.has_key((self.control_name, object_)):
                connection = AVCCD.connections[(self.control_name,
                        object_)]
            else:
                connection = AVCCD.connections[(self.control_name,
                        object_)] = Connection(value)
                return

        if value == connection.control_value:
            return

        if connection.control_type in (list, dict):
            connection.control_value = copy.deepcopy(value)
        else:
            connection.control_value = value

        if setter and connection.set_handler:
            connection.set_handler(value)

        if AVCCD.view_period != 0.0:
            AVCCD.connections_updates[connection] = setter
            return

        for wal_widget in connection.wal_widgets:
            if wal_widget != setter:
                wal_widget.write(value)

    def __delete__(self, instance):
        """Cogets cannot be deleted"""

        raise Error, 'Trying to delete ' + str(self)\
             + ': Cogets cannot be deleted.'


class Widget:

    """Widget Abstraction Layer abstract class"""

    def __init__(
        self,
        connection,
        widget,
        allowed_types=None,
        ):

        if allowed_types and not connection.control_type in allowed_types:
            raise Error, \
                "Control type '%s' not supported with '%s' widget"\
                 % (connection.control_type.__name__,
                    widget.__class__.__name__)

        self.connection = connection
        self.widget = widget

        self.connect_delete(widget, self.delete)

    def read(self):
        """Read value from widget"""
        raise Error, \
            'Method "read" of abstract class Widget is undefined'

    def write(self, value):
        """Write value into widget"""
        raise Error, \
            'Method "write" of abstract class Widget is undefined'

    def value_changed(self, *args):
        """widget value changed handler"""

        Coget.__set__(self.connection.coget, '', self.read(), self,
                      self.connection)

    def delete(self, *args):
        """delete widget from connection"""

        self.connection.remove_widget(self)


class ListTreeView(Widget, REAL.ListTreeView):
    """ ListTreeView widget abstractor"""
    def __init__(self, connection, listtreeview):
        """Common init operations for ListView and TreeView abstractors"""

        Widget.__init__(self, connection, listtreeview, (dict, ))

        head = connection.control_value.get('head', None)
        if head and type(head) != list:
            raise Error, \
                "%s widget do not allow '%s' type as header, use a list."\
                 % (listtreeview.__class__.__name__,
                    type(head).__name__)

        self.cols_num = len(self.row_types)

        if head and len(head) != self.cols_num:
            raise Error, \
                '%s widget require header lenght equal to data row size.'\
                 % listtreeview.__class__.__name__

        REAL.ListTreeView.__init__(self)

        head = self.connection.control_value.get('head', None)
        if head:
            map(self.append_column, range(len(self.row_types)), head)
        else:
            map(lambda col_num: self.append_column(col_num, ''),
                range(len(self.row_types)))


class Button(REAL.Button, Widget):

    """Button widget abstractor"""

    def __init__(self, connection, button):

        Widget.__init__(self, connection, button, (bool, ))

        REAL.Button.__init__(self)


class Calendar(REAL.Calendar, Widget):

    """Calendar widget abstractor"""

    def __init__(self, connection, calendar):

        Widget.__init__(self, connection, calendar, (list, tuple))

        REAL.Calendar.__init__(self)


class ComboBox(REAL.ComboBox, Widget):

    """ComboBox widget abstractor"""

    def __init__(self, connection, combobox):

        Widget.__init__(self, connection, combobox, (int, ))

        REAL.ComboBox.__init__(self)


class ColorChooser(REAL.ColorChooser, Widget):

    """ColorChooser widget abstractor"""

    def __init__(self, connection, colorchooser):

        Widget.__init__(self, connection, colorchooser, (list, tuple))

        REAL.ColorChooser.__init__(self)


class Entry(REAL.Entry, Widget):

    """Entry widget abstractor"""

    def __init__(self, connection, entry):

        Widget.__init__(self, connection, entry, (float, int, str))

        REAL.Entry.__init__(self)

    def read(self):
        """Get Entry value"""

        return self.connection.control_type(REAL.Entry.read(self))


class Label(REAL.Label, Widget):

    """Label widget abstractor"""

    def __init__(self, connection, label):

        Widget.__init__(self, connection, label)

        if connection.control_type in (
            bool,
            float,
            int,
            list,
            str,
            tuple,
            dict,
            ):
            self.is_object = False
            control_value = connection.control_value
        else:
            self.is_object = True
            control_value = connection.control_value.__dict__

        self.format = str(self.read())

        try:
            if connection.control_type == list:
                junk = self.format % tuple(control_value)
            elif connection.control_type == dict:
                junk = self.format % control_value
                if junk == self.format:
                    raise
            else:
                junk = self.format % control_value
            if AVCCD.verbosity > 2:
                print '    valid format string: "' + self.format + '"'
        except:
            if AVCCD.verbosity > 2:
                if self.format:
                    print '    invalid format string: "' + self.format\
                         + '"'
                else:
                    print '    no format string'
            self.format = None

        REAL.Label.__init__(self)

    def read(self):
        """Get value from Label"""

        if self.is_object:
            return REAL.Label.read(self)

        try:
            return self.connection.control_type(eval(REAL.Label.read(self)))
        except:

            return REAL.Label.read(self)

    def write(self, value):
        """Set text into Label"""

        if self.format:
            if self.is_object:
                REAL.Label.write(self, self.format % value.__dict__)
            else:
                if type(value) == list:
                    value = tuple(value)
                REAL.Label.write(self, self.format % value)
        else:
            REAL.Label.write(self, str(value))


class ListView(REAL.ListView, ListTreeView):

    """ListView widget abstractor"""

    def __init__(self, connection, listview):

        body = connection.control_value.get('body', None)
        if type(body[0]) == list:
            self.row_types = [type(row) for row in body[0]]
        else:
            self.row_types = [type(body[0])]

        ListTreeView.__init__(self, connection, listview)

        REAL.ListView.__init__(self)


class ProgressBar(REAL.ProgressBar, Widget):

    """ProgressBar widget abstractor"""

    def __init__(self, connection, progressbar):

        Widget.__init__(self, connection, progressbar, (float, int))

        REAL.ProgressBar.__init__(self)

    def read(self):
        """Get Entry value"""

        return self.connection.control_type(REAL.Entry.read(self))


class RadioButton(REAL.RadioButton, Widget):

    """RadioButton widget abstractor"""

    def __init__(self, connection, radiobutton):

        Widget.__init__(self, connection, radiobutton, (int, ))

        REAL.RadioButton.__init__(self)


class Slider(REAL.Slider, Widget):

    """Slider widget abstractor"""

    def __init__(self, connection, slider):

        Widget.__init__(self, connection, slider, (float, int))

        REAL.Slider.__init__(self)

    def read(self):
        """Get Slider value"""

        return self.connection.control_type(REAL.Slider.read(self))


class SpinButton(REAL.SpinButton, Widget):

    """SpinButton widget abstractor"""

    def __init__(self, connection, spinbutton):

        Widget.__init__(self, connection, spinbutton, (float, int))

        REAL.SpinButton.__init__(self)

    def read(self):
        """Get spinbutton value"""

        return self.connection.control_type(REAL.SpinButton.read(self))


class StatusBar(REAL.StatusBar, Widget):

    """StatusBar widget abstractor"""

    def __init__(self, connection, statusbar):

        Widget.__init__(self, connection, statusbar, (str, ))

        REAL.StatusBar.__init__(self)


class TextView(REAL.TextView, Widget):

    """TextView widget abstractor"""

    def __init__(self, connection, textview):

        Widget.__init__(self, connection, textview, (str, ))

        REAL.TextView.__init__(self)


class ToggleButton(REAL.ToggleButton, Widget):

    """ToggleButton widget abstractor"""

    def __init__(self, connection, togglebutton):

        Widget.__init__(self, connection, togglebutton, (bool, ))

        REAL.ToggleButton.__init__(self)


class TreeView(REAL.TreeView, ListTreeView):

    """TreeView widget abstractor"""

    def __init__(self, connection, treeview):

        body = connection.control_value.get('body', None)
        self.row_types = [type(row) for row in body.itervalues().next()]

        ListTreeView.__init__(self, connection, treeview)

        REAL.TreeView.__init__(self)

    def write(self, value):
        """Set values displayed by widget"""

        if value.has_key('head'):
            self.write_head(value['head'])

        body = value['body']

        node_ids = body.keys()
        nodes = zip(map(lambda id: map(int, id.split('.')), node_ids), node_ids)
        nodes.sort()

        current_depth = 1
        parents = [self.root_node()]
        last_node = None
        for node in nodes:
            node_depth = len(node[0])
            if node_depth > current_depth:
                parents.append(last_node)
                current_depth = node_depth
            elif node_depth < current_depth:
                last_node = parents.pop(-1)
                current_depth = node_depth
            last_node = self.add_node(parents[-1], last_node,
                    current_depth, body[node[1]])


def listtreeview(connection, treeview):
    """
  Route real tree view widgets supporting both list and tree data structures
  to the abstract widgets supporting only list or tree data.
  """

    body_type = type(connection.control_value.get('body', None))
    if body_type == list:
        return ListView(connection, treeview)
    elif body_type == dict:
        return TreeView(connection, treeview)
    else:
        raise Error, "%s widget do not allow '%s' type as data,"\
             + 'use a list for tabular data or a dictionary for tree data.'\
             % (treeview.__class__.__name__, type(body).__name__)





#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : A counter with count speed control (wx)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	24-Nov-2007
# .copyright  : (c) 2007 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


import wx				# wx tool kit bindings
from wx import xrc			# xml resource support

from avc import *			# AVC

WXGLADE_XML = 'wx_counter.xrc'		# GUI wxGlade descriptor
LOW_SPEED = 0.5				#--
HIGH_SPEED = 0.1			#- low and high speed period (ms)


class ExampleGUI(wx.PySimpleApp):
  "Counter GUI creation"

  def __init__(self):

    # init wx application base class
    wx.PySimpleApp.__init__(self)

    # create GUI
    xml_resource = xrc.XmlResource(WXGLADE_XML)
    self.root = xml_resource.LoadFrame(None,'frame_1')
    self.root.Show()

    # timer
    self.timer1 = None


  def timer(self,period,function):
    "Start a wx timer calling back 'function' every 'period' seconds."
    if not self.timer1:
      self.timer1 = wx.Timer(self.root,wx.NewId())
      self.root.Bind(wx.EVT_TIMER,function,self.timer1)
    self.timer1.Start(period * 1000,oneShot=True)


class ExampleMain(AVC):
  """
  A counter displayed in a Label widget whose count speed can be
  accelerated by checking a check button.
  """

  def __init__(self,gui):

    # save gui
    self.gui = gui

    # the counter variable and its speed status
    self.counter = 0
    self.high_speed = False

    # start incrementer timer
    self.gui.timer(LOW_SPEED,self.incrementer)


  def incrementer(self,event):
    """
    Counter incrementer: increment period = LOW_SPEED, if high speed is False,
    increment period = HIGH_SPEED otherwise. Return False to destroy previous
    timer.
    """
    self.counter += 1
    if self.high_speed:
      period = HIGH_SPEED
    else:
      period = LOW_SPEED
    self.gui.timer(period,self.incrementer)

  def high_speed_changed(self,value):
    "Notify change of counting speed to terminal"
    if value:
      print 'counting speed changed to high'
    else:
      print 'counting speed changed to low'


#### MAIN

example_gui = ExampleGUI()		# create the application GUI
example = ExampleMain(example_gui)	# instantiate the application
example.avc_init()			# connect widgets with variables
example_gui.MainLoop()			# run wx event loop until quit

#### END



#!/usr/bin/python
# -*- coding: utf-8 -*-
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : Calendar widget connected to label widget (wx)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	31-Jan-2013
# .copyright  : (c) 2013 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


import wx				    # wx tool kit bindings
from wx import xrc			# xml resource support

from avc import *			# AVC for wx

WXGLADE_XML = 'wx_calendar.xrc'		# GUI wxGlade descriptor


class Example(wx.PySimpleApp,AVC):
  """
  Calendar widget connected to label widget (GTK)
  """

  def __init__(self):

    # init wx application base class
    wx.PySimpleApp.__init__(self)

    # create GUI
    xml_resource = xrc.XmlResource(WXGLADE_XML)
    self.root = xml_resource.LoadFrame(None,'frame_1')
    self.root.Show()
     
    # all types of connected variables
    self.calendar_value = (2000,1,1)


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
example.MainLoop()		 	# run wx event loop until quit

#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : list tree view widget display capabilities (Qt4)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	9-Dec-2008
# .copyright  : (c) 2008 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


from PyQt4.QtCore import *		# Qt core
from PyQt4.QtGui import *		# Qt GUI interface
from PyQt4.uic import *			# ui files realizer

import copy				# object cloning support
import sys				# system support

from avc import *			# AVC

UI_FILE = 'qt4_listtreeview.ui'		# qt ui descriptor

UPDATE_PERIOD = 2000			# ms


class Example(QApplication,AVC):
  """
  Showcase of display capabilities for the list tree view widget
  """

  def __init__(self):

    # create GUI
    QApplication.__init__(self,sys.argv)
    self.root = loadUi(UI_FILE)
    self.root.show()

    # connected variables
    self.list = {'head':['col1 int','col2 str'], \
      'body':[[1,'one'],[2,'two'],[3,'three']]}
    self.list_work = copy.deepcopy(self.list)
    self.tree = {'head':['col1 int','col2 str'],'body':{ \
      # root rows
      '1':[1,'one'], \
      '2':[2,'two'], \
      # children of root row '1'
      '1.1':[11,'one one'], \
      '1.2':[12,'one two'], \
      # children of root row '2'
      '2.1':[21,'two one'], \
      '2.2':[22,'two two']}}

    # start variables update
    update = self.update()
    self.timer1 = QTimer()
    QObject.connect(self.timer1,SIGNAL("timeout()"),update.next)
    self.timer1.start(UPDATE_PERIOD)


  def update(self):
    """
    Tabular data rows data are rolled down.
    """
    rows_num = len(self.list['body'])
    while True:
      # save last row of data
      last_row = self.list_work['body'][-1]
      # shift down one position each data row
      for i in range(1,rows_num): 
        self.list_work['body'][-i] = \
          self.list_work['body'][-1-i]
      # copy last row into first position
      self.list_work['body'][0] = last_row
      # copy working copy into connected variable
      self.list = self.list_work
      yield True


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
example.exec_()				# run Qt event loop until quit

#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : A spin control replicated into a text label (Tk)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	30-May-2007
# .copyright  : (c) 2007 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


from Tkinter import *			# Tk interface

from avc import *			# AVC

TCL_FILE = 'tk_spinbox.tcl'		# GUI description as tcl script


class Example(AVC):
  """
  A spin box whose value is replicated into a label
  """

  def __init__(self):



    # create GUI
    self.root = Tk()
    self.root.eval('set argc {}; set argv {}; proc ::main {argc argv} {};')
    self.root.tk.evalfile(TCL_FILE)

    # terminate program at toplevel window destroy: connect toplevel
    # destroy signal to termination handler.
    self.root.bind_class('Toplevel','<Destroy>',lambda event: self.root.quit())

    # the variable holding the spin control value
    self.spin_value = 0


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
mainloop()			 	# run Tk event loop until quit

#### END



# .+
# .context    : Application View Controller
# .title      : AVC Tk bindings
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	7-Nov-2006
# .copyright  : (c) 2006-2008 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with AVC.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


#### IMPORT REQUIRED MODULES

import Tkinter				# Tk interface


#### GENERAL ABSTRACTION METHODS

## redifinition of BaseWidget __init__ in Tkinter with a "monkey patch".
# The same as the original, but without any call to tcl interpreter.
# Needed to instantiate the python widget of a already existing tcl widget.

def _dummy_call(*args):
  return

def _basewidget_new_init(self, master, widgetName, cnf={}, kw={}, extra=()):
  """
  Construct a widget with the parent widget MASTER, a name WIDGETNAME
  and appropriate options.
  """
  if kw:
    cnf = Tkinter._cnfmerge((cnf, kw))
  self.widgetName = widgetName
  Tkinter.BaseWidget._setup(self, master, cnf)



def toplevel_widgets():
  "Return the list of all top level widgets"
  return [Tkinter._default_root]

def complete_widget_tree(widget):
  """
  Since direct usage of the tcl interpreter (call to "eval","evalfile" or
  "call") do not create any widget instance in python, complete the python
  widget tree instantiating the corresponding widget class from Tkinter
  for each widget created by the tcl interpreter.
  Do the work by visiting in deep first mode the widget tree with the tcl
  interpreter commands.
  Also build the list of all top level widgets.
  """
  children_path = widget.tk.eval('winfo children ' + str(widget)).split()
  for child_path in children_path:
    child_name = child_path.split('.')[-1]
    # if python widget do not exists, create it.
    if not widget.children.has_key(child_name):
      class_name = widget.tk.eval('winfo class ' + child_path)
      eval('Tkinter.' + class_name + '(widget,name="' + child_name + '")')
    # if widget is a toplevel window, append it to top level widgets list.
    ##if child_path == widget.tk.eval('winfo toplevel ' + child_path):
    ##  self._toplevel_widgets += [widget.nametowidget(child_path)]
    # go deep into widgets tree
    for child in widget.children.values():
      complete_widget_tree(child)

def init(core):
  "Do init specific for this widget toolkit"
  # replace Tkinter basewidget init, with the one not calling tcl interpreter
  basewidget_original_init = Tkinter.BaseWidget.__init__
  Tkinter.BaseWidget.__init__ = _basewidget_new_init
  toplevel_original_title = Tkinter.Wm.title
  Tkinter.Wm.title = _dummy_call
  # complete the python widget tree, if necessary
  complete_widget_tree(toplevel_widgets()[0])
  # restore original basewidget init
  Tkinter.BaseWidget.__init__ = basewidget_original_init
  Tkinter.Wm.title = toplevel_original_title
  # mapping between the real widget and the wal widget
  core['avccd'].widget_map = { \
    Tkinter.Button:	core['Button'],\
    Tkinter.Checkbutton:core['ToggleButton'], \
    Tkinter.Entry:	core['Entry'], \
    Tkinter.Label:	core['Label'], \
    Tkinter.Radiobutton:core['RadioButton'], \
    Tkinter.Scale:	core['Slider'], \
    Tkinter.Spinbox:	core['SpinButton'],
    Tkinter.Text:	core['TextView']}
  # get toolkit version
  core['avccd'].toolkit_version = Tkinter.__version__


def widget_children(widget):
  "Return the list of all children of the widget"
  return widget.children.values()

def widget_name(widget):
  """
  Return the widget name. Try first to get avc_name attribute. If this
  fails, get Tk widget name keeping the last part only.
  """
  try:
    return widget.avc_name
  except:
    return str(widget).split('.')[-1]

def timer(function,period):
  "Call given function, reschedule it after return"
  function()
  return toplevel_widgets()[0].after(int(period * 1000.0),
    timer,function,period)


#### WIDGETS ABSTRACTION LAYER (widget toolkit side)

class Widget:
  "Tk Widget Abstraction Layer abstract class"

  def connect_delete(self,widget,delete_method):
    "Connect widget delte method to destroy event"
    widget.bind("<Destroy>",delete_method)


class ListTreeView(Widget):
  "Tk has no list view and tree view support"
  pass


class Button(Widget):
  "Tk Button widget abstractor"

  def __init__(self):

    # create button press status variable
    self.widget.value = False

    # connect relevant signals
    self.widget.bind("<ButtonPress-1>",
      lambda dummy: self.connection.coget.__set__(
        '',True,self.widget,self.connection))
    self.widget.bind("<ButtonRelease-1>",
      lambda dummy: self.connection.coget.__set__(
        '',False,self.widget,self.connection))


  def read(self):
    "Get button status"
    return self.widget.value

  def write(self,value):
    "Set button status"
    self.widget.value = value


class Calendar(Widget):
  "Widget not in tk"

  def __init__(self):
      pass


class ComboBox(Widget):
  "Tk no combo box support"
  pass


class ColorChooser(Widget):
  "Widget not in tk"

  def __init__(self):
      pass


class Entry(Widget):
  "Tk Entry widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.bind("<Return>",self.value_changed)

  def read(self):
    "Get text from Entry"
    return self.widget.get()
    
  def write(self,value):
    "Set text into Entry"
    self.widget.delete(0,Tkinter.END)
    self.widget.insert(0,str(value))

 
class Label(Widget):
  "Tk Label widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get value into Label"
    return self.widget.cget('text')

  def write(self,value):
    "Set text into Label"
    self.widget.config(text=value)


class ListView(ListTreeView):
  "Tk has no list view support"
  pass


class ProgressBar(Widget):
  "Tk no prograss bar support"
  pass

class RadioButton(Widget):
  "Tk RadioButton widget abstractor"

  def __init__(self):

    # if not yet done, get existing (by default) variable with pressed
    # button index.
    if not hasattr(self.connection,'active_index_name'):
      self.connection.active_index_name = str(self.widget.cget("variable"))
    
    # connect relevant signals
    self.widget.bind("<ButtonRelease-1>",self.value_changed)


  def read(self):
    "Get index of activated button"
    return int(str(self.widget.getvar(self.connection.active_index_name)))

  def write(self,value):
    "Set activate button indexed by value"
    self.widget.setvar(self.connection.active_index_name,value)


class Slider(Widget):
  "Tk Scale widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.config(command=self.value_changed)
 
  def read(self):
    "Get Slider value"
    return self.widget.get()

  def write(self,value):
    "Set Slider value"
    self.widget.set(str(value))


class SpinButton(Widget):
  "Tk SpinButton widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.bind("<Return>",self.value_changed)
    self.widget.config(command=self.value_changed)

  def read(self):
    "Get spin button value"
    return self.widget.get()

  def write(self,value):
    "Set spin button value"
    self.widget.delete(0,Tkinter.END)
    self.widget.insert(0,str(value))
 

class StatusBar(Widget):
  "Tk no status bar support"
  pass


class TextView(Widget):
  "Tk TextView widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.bind("<Return>",self.value_changed)

  def read(self):
    "Get text from TextView"
    return self.widget.get("1.0",Tkinter.END)
    
  def write(self,value):
    "Set text into TextView"
    self.widget.delete("1.0",Tkinter.END)
    self.widget.insert("1.0",str(value))


class ToggleButton(Widget):
  "Tk ToggleButton widget abstractor"

  def __init__(self):

    # get and save button value variable name
    self.value_name = str(self.widget.cget('variable'))

    # connect relevant signals
    self.widget.bind("<ButtonRelease-1>",self.value_changed)


  def read(self):
    "Get button status"
    return bool(int(self.widget.getvar(self.value_name)))

  def write(self,value):
    "Set button status"
    self.widget.setvar(self.value_name,int(value))


class TreeView(ListTreeView):
  "Tk has no tree view support"
  pass

#### END




#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : Random generation of count down windows (Qt4)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Torino - Italy
# .creation   :	13-May-2008
# .copyright  : (c) 2008 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


from PyQt4.QtCore import *		# Qt core
from PyQt4.QtGui import *		# Qt GUI interface
from PyQt4.uic import *			# ui files realizer
import sys				# system support

from avc import *			# AVC

from random import randint		# random integer generator

UI_MAIN = 'qt4_countdown_main.ui'	# qt ui descriptor for main window
UI_CD = 'qt4_countdown.ui'		# qt ui descriptor for countdown window
TOPLEVEL_NAME = 'countdown'		# name of the top level widget
COUNTDOWN_PERIOD = 500			# count down at 2 unit per second
MAX_CREATION_PERIOD = 4000		# create a new count down at 1/2 this


class Countdown(AVC):
  """
  A countdown counter displayed in a Label widget. Count starts at given
  value. When count reaches zero the counter and its GUI are destroyed.
  """

  def __init__(self,count_start=10):

    # create GUI
    self.root = loadUi(UI_CD)
    self.root.show()

    # init the counter variable 
    self.counter = count_start

    # connect counter variable with label widget
    self.avc_connect(self.root)

    # start count down
    self.timer = QTimer(self.root)
    self.root.connect(self.timer,SIGNAL("timeout()"),self.decrementer)
    self.timer.start(COUNTDOWN_PERIOD)


  def decrementer(self):
    "Counter decrementer. Return False to destroy previous timer."
    self.counter -= 1
    # if counter reached zero, destroy this countdown and its GUI
    if not self.counter:
      self.timer.stop()
      del self.timer
      self.root.close()


class Example(QApplication,AVC):
  """
  Continuously create at random intervals windows with a countdown from 10 to 0.
  When a countdown reaches zero, its window is destroyed. Also create a main
  window with a "close all" button.
  """

  def __init__(self):

    # create main window
    QApplication.__init__(self,sys.argv)
    self.root = loadUi(UI_MAIN)
    self.root.show()
    
    # close all button connected variable
    self.close_all = False

    # start count down
    self.timer = QTimer(self)
    self.connect(self.timer,SIGNAL("timeout()"),self.new_countdown)
    self.timer.start(randint(1,MAX_CREATION_PERIOD))


  def new_countdown(self,count_start=10):
    "Create a new countdown"

    # create a new countdown
    Countdown(count_start)

    # autocall after a random delay
    self.timer.stop()
    self.timer.start(randint(1,MAX_CREATION_PERIOD)) 


  def close_all_changed(self,value):
    "Terminate program at 'close all' button pressing"
    self.quit()


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
example.exec_()				# run Qt event loop until quit

#### END



# .+
# .context    : Application View Controller
# .title      : AVC GTK+ bindings
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	7-Nov-2006
# .copyright  :	(c) 2006-2011 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with AVC.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


#### IMPORT REQUIRED MODULES

import gtk			#--
import gobject			#- gimp tool kit bindings

import string			# string operations

#### GENERAL ABSTRACTION METHODS

def toplevel_widgets():
  "Return the list of all top level widgets"
  return gtk.window_list_toplevels()

def init(core):
  "Do init specific for this widget toolkit"
  # mapping between the real widget and the wal widget
  core['avccd'].widget_map = { \
  gtk.Button:		    core['Button'], \
  gtk.Calendar:	    	core['Calendar'], \
  gtk.CheckButton:	    core['ToggleButton'], \
  gtk.ComboBox:	    	core['ComboBox'],\
  gtk.ColorSelection:	core['ColorChooser'],\
  gtk.Entry:    		core['Entry'], \
  gtk.Label:	    	core['Label'], \
  gtk.ProgressBar:	    core['ProgressBar'], \
  gtk.RadioButton:  	core['RadioButton'], \
  gtk.HScale:		    core['Slider'], \
  gtk.SpinButton:	    core['SpinButton'], \
  gtk.Statusbar:    	core['StatusBar'], \
  gtk.TextView:		    core['TextView'], \
  gtk.ToggleButton:	    core['ToggleButton'], \
  gtk.TreeView:		    core['listtreeview'], \
  gtk.VScale:		    core['Slider']}
  # get toolkit version
  core['avccd'].toolkit_version = '.'.join(map(str,gtk.gtk_version))

def widget_children(widget):
  "Return the list of all children of the widget"
  # widgets that are not a subclass of gtk.Container have no children.
  if isinstance(widget,gtk.Container):
    return widget.get_children()
  else:
    return []

def widget_name(widget):
  "Return the widget name"
  return widget.get_name()

def timer(function,period):
  """
  Start a GTK timer calling back 'function' every 'period' seconds.
  Return timer id.
  """
  return gobject.timeout_add(int(period * 1000.0),timer_wrap,function)

def timer_wrap(function):
  "Call given function and return True to keep timer alive"
  function()
  return True


#### WIDGETS ABSTRACTION LAYER (widget toolkit side)

class Widget:
  "GTK Widget Abstraction Layer abstract class"

  def __init__(self,allowed_types=None):
    # check for supported control type
    if allowed_types and not self.connection.control_type in allowed_types:
      raise error, "Control type '%s' not supported with '%s' widget" % \
        (self.connection.control_type.__name__,self.widget.__class__.__name__)

  def connect_delete(self,widget,delete_method):
    "Connect widget delete method to destroy event"
    widget.connect("destroy",delete_method)


class ListTreeView(Widget):
  "Support to ListView and TreeView abstractors"

  def __init__(self):
    "Init operations common to ListView and TreeView"
    pass

  def append_column(self,col_num,text):
   "Append a column to the TreeView"
   self.widget.append_column(gtk.TreeViewColumn(
     text,gtk.CellRendererText(),text=col_num))
   

class Button(Widget):
  "GTK Button real widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.connect("pressed",self.value_changed)
    self.widget.connect("released",self.value_changed)

  def read(self):
    "Get button status"
    if self.widget.state == gtk.STATE_ACTIVE:
      return True
    return False

  def write(self,value):
    "Set button status"
    if value:
      self.widget.set_state(gtk.STATE_ACTIVE)
    else:
      self.widget.set_state(gtk.STATE_NORMAL)


class Calendar(Widget):
  "GTK Calendar real widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.connect("day-selected",self.value_changed)

  def read(self):
    "Get calendar date"
    date = self.widget.get_date()
    # make month number starting from 1
    return (date[0],date[1]+1,date[2])

  def write(self,value):
    "Set calendar date"
    # make month number starting from 0
    self.widget.select_month(value[1]-1,value[0])
    self.widget.select_day(value[2])


class ColorChooser(Widget):
  "GTK ColorChooser real widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.connect("color-changed",self.value_changed)

  def read(self):
    "Get current color"
    color = self.widget.get_current_color()
    alpha = self.widget.get_current_alpha()
    # color from int to float 0.0-1.0
    return (color.red/65535.,color.green/65535.,color.blue/65535.,alpha/65535.)

  def write(self,value):
    "Set current color"
    self.widget.set_current_color(gtk.gdk.Color(
      int(value[0]*65535),int(value[1]*65535),int(value[2]*65535)))
    self.widget.set_current_alpha(int(value[3] * 65535))


class ComboBox(Widget):
  "GTK ComboBox widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.connect("changed",self.value_changed)

  def read(self):
    "Get index of selected item"
    return self.widget.get_active()

  def write(self,value):
    "Set selected item by its index value"
    self.widget.set_active(value)


class Entry(Widget):
  "GTK Entry widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.connect("activate",self.value_changed)

  def read(self):
    "Get text from Entry"
    return self.widget.get_text()
    
  def write(self,value):
    "Set text into Entry"
    self.widget.set_text(str(value))
 

class Label(Widget):
  "GTK Label widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get value from Label"
    return self.widget.get_label()

  def write(self,value):
    "Set text into Label"
    self.widget.set_label(value)


class ListView(ListTreeView):
  "GTK TreeView widget abstractor"

  def __init__(self):
    # prepare data model.
    self.model = gtk.ListStore(*self.row_types)
    # set model to widget
    self.widget.set_model(self.model)

    # connect relevant signals
    self.model.connect("row-deleted",self.value_changed)
    self.model.connect("row-changed",self.value_changed)


  def read(self):
    "Get values displayed by widget"
    # get head
    head = map(gtk.TreeViewColumn.get_title,self.widget.get_columns())
    # get data rows
    body = []
    self.model.foreach(
      lambda model,path,iter,body: body.append(
        list(model.get(iter,*range(self.cols_num)))),body)
    # return
    return {'head': head,'body': body}


  def write(self,value):
    "Set values displayed by widget"
    # set header
    if value.has_key('head'):
      map(gtk.TreeViewColumn.set_title,self.widget.get_columns(),value['head'])
    # set data rows
    body = value['body']
    self.model.clear()
    if type(body[0]) == list:
      for row in body:
        self.model.append(row)
    else:
      for row in body:
        self.model.append([row])


class ProgressBar(Widget):
  "GTK ProgressBar widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get progress bar position"
    return self.widget.get_fraction()
    
  def write(self,value):
    "Set progress bar position"
    # negative values pulse the bar, positive values position the bar. 
    if value < 0:
      self.widget.pulse()
    else:
      self.widget.set_fraction(value)
 

class RadioButton(Widget):
  "GTK RadioButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.connect("clicked",self.value_changed)

  def read(self):
    "Get index of activated button"
    button = self.widget
    buttons = button.get_group()
    for index,rbutton in enumerate(buttons):
      if rbutton.get_active():
        break
    index = len(buttons) - index - 1
    return index

  def write(self,value):
    "Set activate button indexed by value"
    button = self.widget
    rbuttons = button.get_group()
    rbutton = rbuttons[len(rbuttons) - value - 1]
    rbutton.set_active(True)


class Slider(Widget):
  "GTK Slider widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.connect("value_changed",self.value_changed)

  def read(self):
    "Get Slider value"
    return self.widget.get_value()

  def write(self,value):
    "Set Slider value"
    self.widget.set_value(value)


class SpinButton(Widget):
  "GTK SpinButton widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.connect("value_changed",self.value_changed)

  def read(self):
    "Get spinbutton value"
    return self.widget.get_value()

  def write(self,value):
    "Set spinbutton value"
    self.widget.set_value(value)


class StatusBar(Widget):
  "GTK StatusBar widget abstractor"

  def __init__(self):
    pass

  def write(self,value):
    "Set StatusBar value"
    self.widget.pop(1)
    self.widget.push(1,value)


class TextView(Widget):
  "GTK TextView widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.get_buffer().connect("changed",self.value_changed)

  def read(self):
    "Get text from TextView"
    textbuf = self.widget.get_buffer()
    return textbuf.get_text(textbuf.get_start_iter(),textbuf.get_end_iter())
    
  def write(self,value):
    "Set text into TextView"
    self.widget.get_buffer().set_text(str(value))


class ToggleButton(Widget):
  "GTK ToggleButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.connect("clicked",self.value_changed)

  def read(self):
    "Get button status"
    return self.widget.get_active()

  def write(self,value):
    "Set button status"
    self.widget.set_active(value)


class TreeView(ListTreeView):
  "GTK TreeView widget abstractor"

  def __init__(self):
    # prepare data model.
    self.model = gtk.TreeStore(*self.row_types)
    # set model to widget
    self.widget.set_model(self.model)

    # connect relevant signals
    self.model.connect("row-deleted",self.value_changed)
    self.model.connect("row-changed",self.value_changed)


  def read(self):
    "Get values displayed by widget"
    # get head
    head = map(gtk.TreeViewColumn.get_title,self.widget.get_columns())
    # get data rows
    body = {}
    # recursive depth first tree data node reader
    def read_node(node_iter,parent_id):
      node_id = string.join(
        map(lambda x: str(x+1),self.model.get_path(node_iter)),'.')
      body[node_id] = list(self.model.get(node_iter,*range(self.cols_num)))
      child_iter = self.model.iter_children(node_iter)
      while child_iter:
        read_node(child_iter,node_id)
        child_iter = self.model.iter_next(child_iter)
    node_iter = self.model.get_iter_first()
    while node_iter:
      read_node(node_iter,None)
      node_iter = self.model.iter_next(node_iter)
    # return
    return {'head': head,'body': body}

  def write_head(self,head):
    "Write header"
    map(gtk.TreeViewColumn.set_title,self.widget.get_columns(),head)

  def root_node(self):
    "Return the root node of the tree"
    return None

  def add_node(self,parent,last_node,current_depth,data):
    "Add current node to the tree"
    return self.model.append(parent,data)
      

#### END



#!/usr/bin/env jython
# .+
# .context    : Application View Controller
# .title      : A counter with count speed control (Swing),
#		GUI programmatically generated
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Torino - Italy
# .creation   :	22-Oct-2009
# .copyright  : (c) 2009 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-

from javax import swing			# swing toolkit bindings
from java import awt			# awt toolkit bindings

from avc import *			# AVC for Swing

LOW_SPEED = 500				#--
HIGH_SPEED = 100			#- low and high speed period (ms)


class ExampleGUI:
  "Counter GUI creation"

  def __init__(self):

    # create GUI
    root = swing.JFrame('AVC Swing counter example',size=(350,60),
      defaultCloseOperation = swing.JFrame.EXIT_ON_CLOSE)
    root.layout = awt.FlowLayout()
    root.add(swing.JLabel('counter',name='counter__label',))
    root.add(swing.JCheckBox('high speed',name='high_speed__checkbox'))
    root.show()

    # create a timer for incrementer
    self.timer = swing.Timer(LOW_SPEED,None)
  

class ExampleMain(AVC):
  """
  A counter displayed in a Label widget whose count speed can be
  accelerated by checking a check box.
  """

  def __init__(self,gui):

    # save GUI
    self.gui = gui

    # the counter variable and its speed status
    self.counter = 0
    self.high_speed = False

    # start variable incrementer
    self.gui.timer.actionPerformed = self.incrementer
    self.gui.timer.start()


  def incrementer(self,*args):
    """
    Counter incrementer: increment period = LOW_SPEED, if high speed is False,
    increment period = HIGH_SPEED otherwise.
    """
    self.counter += 1
    if self.high_speed:
      period = HIGH_SPEED
    else:
      period = LOW_SPEED
    self.gui.timer.setDelay(period)

  def high_speed_changed(self,value):
    "Notify change of counting speed to terminal"
    if value:
      print 'counting speed changed to high'
    else:
      print 'counting speed changed to low'


#### MAIN

example_gui = ExampleGUI()		# create the application GUI
example = ExampleMain(example_gui)	# instantiate the application
example.avc_init()			# connect widgets with variables

#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : tree view widget display capabilities (wx)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	25-Dec-2008
# .copyright  : (c) 2008 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


import wx				# wx tool kit bindings
from wx import xrc			# xml resource support

from avc import *			# AVC

import copy				# object cloning support

WXGLADE_XML = 'wx_listtreectrl.xrc'	# GUI wxGlade descriptor

UPDATE_PERIOD = 2000			# ms


class Example(wx.PySimpleApp,AVC):
  """
  Showcase of display capabilities for the list control and tree control widgets
  """

  def __init__(self):

    # init wx application base class
    wx.PySimpleApp.__init__(self)

    # create GUI
    xml_resource = xrc.XmlResource(WXGLADE_XML)
    self.root = xml_resource.LoadFrame(None,'frame_1')
    self.root.Show()

    # connected variables
    self.list = {'head':['col1 int','col2 str'], \
      'body':[[1,'one'],[2,'two'],[3,'three']]}
    self.list_work = copy.deepcopy(self.list)
    self.tree = {'body':{ \
      # root rows
      '1':'one', \
      '2':'two', \
      # children of root row '1'
      '1.1':'one one', \
      '1.2':'one two', \
      # children of root row '2'
      '2.1':'two one', \
      '2.2':'two two'}}

    # start a wx timer calling back 'function' every 'period' seconds."
    self.timer1 = wx.Timer(self.root,wx.NewId())
    self.root.Bind(wx.EVT_TIMER,self.update_wrap,self.timer1)
    self.timer1.Start(UPDATE_PERIOD,oneShot=False)


  def update_wrap(self,event):
    "Discard event argument and call the real update iterator"
    self.update().next()

  def update(self):
    """
    Tabular data rows data are rolled down.
    """
    rows_num = len(self.list['body'])
    while True:
      # save last row of data
      last_row = self.list_work['body'][-1]
      # shift down one position each data row
      for i in range(1,rows_num): 
        self.list_work['body'][-i] = \
          self.list_work['body'][-1-i]
      # copy last row into first position
      self.list_work['body'][0] = last_row
      # copy working copy into connected variable
      self.list = self.list_work
      yield True


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
example.MainLoop()			# run wx event loop until quit

#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : A spin control replicated into a static text (wx)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	24-Nov-2007
# .copyright  : (c) 2007 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


import wx				# wx tool kit bindings

from avc import *			# AVC


class Example(wx.PySimpleApp,AVC):
  """
  A spin control whose value is replicated into a static text
  """

  def __init__(self):

    ## create GUI

    # init wx application base class
    wx.PySimpleApp.__init__(self)

    # create widgets: a top level window, a static text and a spin control.
    root = wx.Frame(None,title='AVC wx spin control example',size=(320,60))
    statictext = wx.StaticText(root,label='%s',name='spin_value__statictext',
      size=(100,20))
    spinctrl = wx.SpinCtrl(root,name='spin_value__spinctrl',size=(100,25))

    # layout the static text and the spin control horizontally into the window
    hsizer = wx.BoxSizer(wx.HORIZONTAL)
    hsizer.Add((20,10),proportion=1)
    hsizer.Add(statictext,proportion=0.5,flag=wx.CENTER)
    hsizer.Add(spinctrl,proportion=0.5,flag=wx.CENTER)
    hsizer.Add((20,10),proportion=1)
    root.SetSizer(hsizer)
    root.Show()


    # the variable holding the spin control value
    self.spin_value = 0


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
example.MainLoop()		 	# run wx event loop until quit

#### END



#!/usr/bin/env jython
# .+
# .context    : Application View Controller
# .title      : A color chooser replicated into a label (Swing),
#		GUI programmatically generated
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	7-Feb-2013
# .copyright  : (c) 2013 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-

from javax import swing			# swing toolkit bindings
from java import awt			# awt toolkit bindings

from avc import *			# AVC for Swing


class Example(AVC):
  """
  A color chooser replicated into a label (Swing),
  """

  def __init__(self):

    # create GUI
    root = swing.JFrame('AVC Swing color chooser example',size=(900,380),
      defaultCloseOperation = swing.JFrame.EXIT_ON_CLOSE)
    root.layout = awt.FlowLayout()
    root.add(swing.JLabel('%s',name='color_value__label'))
    root.add(swing.JColorChooser(name='color_value__colorchooser'))
    root.show()

    # the variable holding the color chooser value
    self.color_value = (0.25,0.5,1.,1.)


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables

#### END



# .+
# .context    : Application View Controller
# .title      : AVC Swing bindings
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	18-Nov-2007
# .copyright  :	(c) 2007-2011 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


#### IMPORT REQUIRED MODULES

from java import awt,beans	# awt and beans tool kit bindings
from javax import swing		# swing tool kit bindings
from java.lang import System	# to get java version

import sys


#### GENERAL ABSTRACTION METHODS

def toplevel_widgets():
  "Return the list of all top level widgets"
  return awt.Frame.getFrames().tolist()

def init(core,*args,**kwargs):
  "Do init specific for this widget toolkit"
  # mapping between the real widget and the wal widget
  core['avccd'].widget_map = { \
    swing.JButton:	core['Button'], \
    swing.JCheckBox:	core['ToggleButton'], \
    swing.JColorChooser:core['ColorChooser'], \
    swing.JComboBox:	core['ComboBox'],\
    swing.JTextField:	core['Entry'], \
    swing.JLabel:	core['Label'], \
    swing.JProgressBar:	core['ProgressBar'], \
    swing.JRadioButton:	core['RadioButton'], \
    swing.JSlider:	core['Slider'], \
    swing.JSpinner:	core['SpinButton'], \
    swing.JTable:	core['ListView'],
    swing.JTextArea:	core['TextView'],
    swing.JToggleButton:core['ToggleButton'],
    swing.JTree:	core['TreeView']}
  # get toolkit version
  core['avccd'].toolkit_version = System.getProperties()['java.runtime.version']

def widget_children(widget):
  "Return the list of all children of the widget"
  # Widgets that are not a subclass of awt.Container have no children.
  if isinstance(widget,awt.Container):
    return widget.components.tolist()
  else:
    return []

def widget_name(widget):
  "Return the widget name"
  widgetname = widget.getName()
  if not widgetname:
    widgetname = '<unnamed>'
  return widgetname

def timer(function,period):
  """
  Start a Swing timer calling back 'function' every 'period' seconds.
  Return timer instance.
  """
  swing_timer = swing.Timer(int(period * 1000.0),None)
  swing_timer.actionPerformed = lambda event: function()
  swing_timer.start()
  return swing_timer


class error(Exception):
  "A generic error exception"
  def __init__(self, value):
    self.value = value
  def __str__(self):
    return repr(self.value)


#### WIDGETS ABSTRACTION LAYER (widget toolkit side)


class Widget:
  "Widget Abstraction Layer abstract class"

  def __init__(self,allowed_types=None):
    # check for supported control type
    if allowed_types and not self.connection.control_type in allowed_types:
      raise error, "Control type '%s' not supported with '%s' widget" % \
        (self.connection.control_type.__name__,self.widget.__class__.__name__)

  def connect_delete(self,widget,delete_method):
    "Connect widget delete method to destroy event"
    # it seems that swing has no destroy signal euivalent for widgets
    pass

class ListTreeView(Widget):
  "Support to ListView and TreeView abstractors"

  def append_column(self,col_num,text):
    "Append a column to the ListTreeView"
    pass


class Button(Widget):
  "Button widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.stateChanged = self.value_changed

  def read(self):
    "Get button status"
    return self.widget.model.isArmed()

  def write(self,value):
    "Set button status"
    self.widget.model.setArmed(value)


class Calendar(Widget):
  "Widget not in swing"

  def __init__(self):
      pass


class ColorChooser(Widget):
  "ColorChooser widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.getSelectionModel().stateChanged = self.value_changed

  def read(self):
    "Get button status"
    color = self.widget.getColor()
    return (color.getRed() / 255., color.getGreen() / 255.,
            color.getBlue() / 255., color.getAlpha() / 255.)

  def write(self,value):
    "Set button status"
    self.widget.setColor(awt.Color(*value))


class ComboBox(Widget):
  "ComboBox widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.addActionListener(ActionListener(self.value_changed))

  def read(self):
    "Get index of selected item"
    return self.widget.getSelectedIndex()

  def write(self,value):
    "Set selected item by its index value"
    self.widget.setSelectedIndex(value)


class Entry(Widget):
  "Entry widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.keyReleased = self.value_changed

  def read(self):
    "Get text from Entry"
    return self.connection.control_type(self.widget.getText())
    
  def write(self,value):
    "Set text into Entry"
    self.widget.setText(str(value))
 

class Label(Widget):
  "Label widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get value from Label"
    return self.widget.text

  def write(self,value):
    "Set text into Label"
    self.widget.text = value


class ListView(ListTreeView):
  "ListView widget abstractor"

  def __init__(self):
    # prepare data model.
    self.model = swing.table.DefaultTableModel(1,len(self.row_types))
    # set model to widget
    self.widget.setModel(self.model)
    # connect relevant signals
    self.model.addTableModelListener(TableModelListener(self.value_changed))

  def read(self):
    "Get values displayed by widget"
    # get head
    head = map(lambda col_num: str(self.model.getColumnName(col_num)),
      range(self.model.getColumnCount()))
    # get data rows
    body = [[str(item) for item in row] for row in self.model.getDataVector()]
    # return
    return {'head': head,'body': body}

  def write(self,value):
    "Set values displayed by widget"
    # set header
    if value.has_key('head'):
      head = value['head']
      col_model = self.widget.getColumnModel()
      map(lambda col_num, col_name:
        col_model.getColumn(col_num).setHeaderValue(col_name),
        range(self.model.getColumnCount()),head)
    else:
      head = [''] * len(self.row_types)  
    # set data rows
    self.model.setDataVector(value['body'],head)


class ProgressBar(Widget):
  "ProgressBar widget abstractor"

  def __init__(self):
    self.minimum = self.widget.getMinimum()
    self.range = self.widget.getMaximum() - self.minimum

  def read(self):
    "Get progress bar position"
    return self.widget.value
    
  def write(self,value):
    "Set progress bar position"
    # negative values pulse the bar, positive values position the bar. 
    if value < 0:
      self.widget.indeterminate = True
    else:
      self.widget.indeterminate = False
      self.widget.value = int(self.minimum + self.range * value)
 

class RadioButton(Widget):
  "RadioButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.stateChanged = self.value_changed

  def read(self):
    "Get index of activated button"
    rbuttons = self.widget.model.getGroup().getElements()
    for index,rbutton in enumerate(rbuttons):
      if rbutton.model.isSelected():
        break
    return index

  def write(self,value):
    "Set activate button indexed by value"
    rbuttons = self.widget.model.getGroup().getElements()
    rbuttons_list = [rbutton for rbutton in rbuttons]
    rbutton = rbuttons_list[value]
    rbutton.model.setSelected(True)


class Slider(Widget):
  "Slider widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.stateChanged = self.value_changed
 
  def read(self):
    "Get Slider value"
    return self.connection.control_type(self.widget.getModel().getValue())

  def write(self,value):
    "Set Slider value"
    self.widget.getModel().setValue(value)


class SpinButton(Widget):
  "SpinButton widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.stateChanged = self.value_changed

  def read(self):
    "Get spinbutton value"
    return self.connection.control_type(self.widget.value)

  def write(self,value):
    "Set spinbutton value"
    self.widget.setValue(value)


class StatusBar(Widget):
  "StatusBar widget abstractor"

  def __init__(self):
    pass

  def set_value(self,value):
    "Set StatusBar value"
    self.widget.pop(1)
    self.widget.push(1,value)


class TextView(Widget):
  "TextView widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.keyReleased = self.value_changed

  def read(self):
    "Get text from TextView"
    return self.widget.getText()
    
  def write(self,value):
    "Set text into TextView"
    self.widget.setText(value)


class ToggleButton(Widget):
  "ToggleButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.stateChanged = self.value_changed

  def read(self):
    "Get button status"
    return self.widget.model.isSelected()

  def write(self,value):
    "Set button status"
    self.widget.model.setSelected(value)


class TreeView(ListTreeView):
  "TreeView widget abstractor"

  def __init__(self):
    # check for allowed control type
    if self.connection.control_value.get('head',None):
      raise error, "%s widget do not support header." \
        % self.widget.__class__.__name__
    key,row = self.connection.control_value.get('body',None).iteritems().next()
    if type(row) == list and not len(row) == 1:
      raise error,"%s widget do not allow data rows with more than one column."\
        % self.widget.__class__.__name__
    # prepare data model.
    self.model =swing.tree.DefaultTreeModel(swing.tree.DefaultMutableTreeNode())
    # set model to widget
    self.widget.setModel(self.model)
    # connect relevant signals
    self.model.addTreeModelListener(TreeModelListener(self.value_changed))
   
  def read(self):
    "Get values displayed by widget"
    # get data rows
    body = {}
    # recursive depth first tree data node reader
    def read_node(node,node_path):
      try:
        body['.'.join(map(str,node_path))] = \
          self.row_types[0](node.toString())
      except:
        if self.row_types[0] == int:
          body['.'.join(map(str,node_path))] = [0]
        elif self.row_types[0] == float:
          body['.'.join(map(str,node_path))] = [0.0]
      children = list(node.children())
      for child_num,child in enumerate(children):
        read_node(child,node_path+[child_num+1])
    root = self.model.getRoot()
    children = list(root.children())
    for child_num,child in enumerate(children):
      read_node(child,[child_num+1])
    # return
    return {'body': body}

  def write_head(self,head):
    "Header not supported"
    pass

  def root_node(self):
    "Return the root node of the tree"
    return self.model.getRoot()

  def add_node(self,parent,last_node,current_depth,data):
    "Add current node to the tree"
    node = swing.tree.DefaultMutableTreeNode(data)
    parent.add(node)
    return node


## support classes

class ActionListener(awt.event.ActionListener):
  "Define status change handler"

  def __init__(self,value_changed):
    self.value_changed = value_changed

  def actionPerformed(self,*args):
    self.value_changed()


class TableModelListener(swing.event.TableModelListener):
  "Define table model change handler"

  def __init__(self,value_changed):
    self.value_changed = value_changed

  def tableChanged(self,*args):
    self.value_changed()


class TreeModelListener(swing.event.TreeModelListener):
  "Define tree model change handler"

  def __init__(self,value_changed):
    self.value_changed = value_changed

  def treeNodesChanged(self,*args):
    self.value_changed()

#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : A table of all supported widget/control type conbinations (wx)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	24-Nov-2007
# .copyright  : (c) 2007 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


import wx				# wx tool kit bindings
from wx import xrc			# xml resource support

from avc import *			# AVC for wx

WXGLADE_XML = 'wx_showcase.xrc'		# GUI wxGlade descriptor
INCREMENTER_PERIOD = 333		# ms


class Example(wx.PySimpleApp,AVC):
  """
  A table of all supported widget/control type combinations
  """

  def __init__(self):

    # init wx application base class
    wx.PySimpleApp.__init__(self)

    # create GUI
    xml_resource = xrc.XmlResource(WXGLADE_XML)
    self.root = xml_resource.LoadFrame(None,'frame_1')
    self.root.Show()

    # the control variables
    self.boolean1 = False
    self.boolean2 = False
    self.index = 0
    self.integer = 0
    self.float = 0.0
    self.progressbar = -1.0
    self.string = ''
    self.textview = ''
    self.status = ''

    # start counter incrementer at low speed
    self.timer = wx.Timer(self.root,wx.NewId())
    self.root.Bind(wx.EVT_TIMER,self.incrementer_wrap,self.timer)
    self.timer.Start(int(INCREMENTER_PERIOD),oneShot=False)
    self.increment = self.incrementer()


  def incrementer_wrap(self,event):
    "Discard event argument and call the real incrementer iterator"
    self.increment.next()

  def incrementer(self,*args):
    """
    Booleans are toggled, radio button index is rotated from first to last,
    integer is incremented by 1, float by 0.5, progress bar is alternatively
    shuttled or incremented from 0 to 100%, string is appended a char
    until maxlen when string is cleared, text view/edit is appended a line
    of text until maxlen when it is cleared. Status bar message is toggled.
    Return True to keep timer alive.
    """
    while True:

      self.boolean1 = not self.boolean1
      yield True

      self.boolean2 = not self.boolean2
      yield True

      if self.index >= 2:
        self.index = 0
      else:
        self.index += 1
      yield True

      self.integer += 1
      yield True

      self.float += 0.5
      yield True

      if self.progressbar >= 0.9999:
        self.progressbar = -1.0
      else:
        self.progressbar += 0.1
      yield True

      if len(self.string) >= 10:
        self.string = ''
      else:
        self.string += 'A'
      yield True

      if len(self.textview) >= 200:
        self.textview = ''
      else:
        self.textview += 'line of text, line of text, line of text\n'
      yield True

      if not self.status:
        self.status = 'status message'
      else:
        self.status = ''
      yield True


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
example.MainLoop()		 	# run wx event loop until quit

#### END



"""
# .+
# .context      : Application View Controller
# .title        : AVC wx bindings
# .kind         : python source
# .author       : Fabrizio Pollastri
# .site         : Torino - Italy
# .creation     : 23-Nov-2007
# .copyright    : (c) 2007-2008 Fabrizio Pollastri
# .license      : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with AVC.  If not, see <http://www.gnu.org/licenses/>.
#
# .-
"""

#### IMPORT REQUIRED MODULES

import wx               # wx tool kit bindings
import wx.calendar      # wx calendar module

import string           # string operations


#### GENERAL ABSTRACTION METHODS

def toplevel_widgets():
  "Return the list of all top level widgets"
  return [wx.GetApp().GetTopWindow()]

def init(core):
  "Do init specific to this widget toolkit"
  # mapping between the real widget and the wal widget
  core['avccd'].widget_map = { \
    wx.BitmapButton:    core['Button'], \
    wx.Button:            core['Button'], \
    wx.calendar.CalendarCtrl:core['Calendar'], \
    wx.CheckBox:        core['ToggleButton'], \
    wx.ColourPickerCtrl:core['ColorChooser'],\
    wx.ComboBox:        core['ComboBox'],\
    wx.Choice:            core['ComboBox'],\
    wx.Gauge:           core['ProgressBar'], \
    wx.ListCtrl:        core['ListView'], \
    wx.RadioBox:        core['RadioButton'], \
    wx.Slider:          core['Slider'], \
    wx.SpinCtrl:        core['SpinButton'], \
    wx.StaticText:      core['Label'], \
    wx.StatusBar:        core['StatusBar'],
    wx.TextCtrl:        core['Entry'],
    wx.ToggleButton:    core['ToggleButton'], \
    wx.TreeCtrl:        core['TreeView']}
  # get toolkit version
  core['avccd'].toolkit_version = wx.VERSION_STRING

def widget_children(widget):
  "Return the list of all children of the widget"
  # Widgets that are not a subclass of gtk.Container have no children.
  if isinstance(widget, wx.Window):
    return widget.GetChildren()
  else:
    return []

def widget_name(widget):
  "Return the widget name"
  return widget.GetName()

def timer(function, period):
  "Create and start a timer calling 'function' every 'period' time"
  first_toplevel = toplevel_widgets()[0]
  timer = wx.Timer(first_toplevel, wx.NewId())
  first_toplevel.Bind(wx.EVT_TIMER, lambda event: function(), timer)
  timer.Start(int(period * 1000.0), oneShot=False)
  return timer


class Error(Exception):
  "A generic error exception"
  def __init__(self, value):
    self.value = value
  def __str__(self):
    return repr(self.value)


#### WIDGETS ABSTRACTION LAYER (widget toolkit side)

class Widget:
  "wx Widget Abstraction Layer abstract class"

  def delete_method_filter(self, event):
    "Ignore destroy events not coming from current widget"
    if self.widget.GetId() == event.GetId():
      self.delete_method()

  def connect_delete(self, widget, delete_method):
    "Connect widget delete method to window destroy event"
    self.delete_method = delete_method
    #widget.Bind(wx.EVT_WINDOW_DESTROY,delete_method)
    widget.Bind(wx.EVT_WINDOW_DESTROY, self.delete_method_filter)


class ListTreeView(Widget):
  "Support to ListView and TreeView abstractors"

  def __init__(self):
    "Init operations common to ListView and TreeView"
    pass


class Button(Widget):
  "wx Button widget abstractor"

  def __init__(self):
    # create button press status variable
    self.widget.value = False
    # connect relevant signals
    self.widget.Bind(wx.EVT_LEFT_DOWN,
      lambda event: wx.CallAfter(
      lambda: self.connection.coget.__set__(
        '',True,self.widget,self.connection))
      or event.Skip())
    self.widget.Bind(wx.EVT_LEFT_UP,
      lambda event: wx.CallAfter(
      lambda: self.connection.coget.__set__(
        '',False,self.widget,self.connection))
      or event.Skip())

  def read(self):
    "Get button status"
    return self.widget.value

  def write(self, value):
    "Set button status"
    self.widget.value = value


class Calendar(Widget):
  "wx Calendar widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.Bind(wx.calendar.EVT_CALENDAR_SEL_CHANGED, self.value_changed)

  def read(self):
    "Get selected date"
    date = self.widget.GetDate()
    # make month number starting from 1
    return (date.GetYear(), date.GetMonth()+1, date.GetDay())

  def write(self, value):
    "Set selected date"
    # change date from yyyymmdd to dd mm-1 yyyy, required by Set
    self.widget.SetDate(wx.DateTime().Set(value[2], value[1]-1, value[0]))


class ColorChooser(Widget):
  "wx ColorChooser widget abstractor"

  def __init__(self):
    # color storage
    self.color = wx.Color()
    # connect relevant signals
    self.widget.Bind(wx.EVT_COLOURPICKER_CHANGED, self.value_changed)

  def read(self):
    "Get selected color"
    color = self.widget.GetColour().Get(True)
    return tuple([float(component / 255.) for component in color])

  def write(self, value):
    "Set selected color"
    self.color.Set(*[int(component * 255) for component in value])
    self.widget.SetColour(self.color)


class ComboBox(Widget):
  "wx ComboBox widget abstractor"

  def __init__(self):
    # connect relevant signals
    if self.widget.__class__ == wx.ComboBox:
      event_type = wx.EVT_COMBOBOX
    else:
      event_type = wx.EVT_CHOICE
    self.widget.Bind(event_type, self.value_changed)

  def read(self):
    "Get index of selected item"
    return self.widget.GetSelection()

  def write(self, value):
    "Set selected item by its index value"
    self.widget.SetSelection(value)


class Entry(Widget):
  "wx Entry widget abstractor"

  def __init__(self):
    # create entry text value variable
    self.widget.value =  self.widget.GetValue()
    # connect relevant signals
    self.widget.Bind(wx.EVT_TEXT, self.value_changed)

  def read(self):
    "Get text from Entry"
    text = self.widget.GetValue()
    # when TextCtrl text is set by program, the event EVT_TEXT is
    # triggered twice: first time with an empty string, second time with
    # the correct value. Substitute first (wrong) value with the old one.
    if text:
      return text
    else:
      return self.widget.value

  def write(self, value):
    "Set text into Entry"
    self.widget.SetValue(str(value))
    self.widget.value = str(value)


class Label(Widget):
  "wx Label widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get value from Label"
    return self.widget.GetLabel()

  def write(self, value):
    "Set text into Label"
    self.widget.SetLabel(value)


class ListView(ListTreeView):
  "wx ListView widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.Bind(wx.EVT_LIST_END_LABEL_EDIT,
      lambda event: wx.CallAfter(self.value_changed) or event.Skip())

  def append_column(self, col_num, text):
    "Append a column to the ListView"
    self.widget.InsertColumn(col_num, text)

  def read(self):
    "Get values displayed by widget"
    # get head
    head = [str(self.widget.GetColumn(col_num).GetText()) \
      for col_num in range(self.widget.GetColumnCount())]
    # get data rows
    body = []
    for row_num in range(self.widget.GetItemCount()):
      row = []
      for col_num in range(self.widget.GetColumnCount()):
        try:
          row.append(self.row_types[col_num](
            self.widget.GetItem(row_num,col_num).GetText()))
        except:
          if self.row_types[col_num] == int:
            row.append(0)
          elif self.row_types[col_num] == float:
            row.append(0.0)
      body.append(row)
    # return
    return {'head': head, 'body': body}

  def write(self, value):
    "Set values displayed by widget"
    # set header
    if value.has_key('head'):
      for col_num in range(self.widget.GetColumnCount()):
        col_item = self.widget.GetColumn(col_num)
        col_item.SetText(value['head'][col_num])
        self.widget.SetColumn(col_num, col_item)
    # set data rows
    body = value['body']
    self.widget.DeleteAllItems()
    if type(body[0]) == list:
      for row_num, row in enumerate(body):
        self.widget.InsertStringItem(row_num,'')
        for col_num, item in enumerate(row):
          self.widget.SetStringItem(row_num, col_num, str(item))
    else:
      for row_num, row in enumerate(body):
        self.widget.InsertStringItem(row_num, str(row))


class ProgressBar(Widget):
  "wx ProgressBar widget abstractor"

  def __init__(self):
    self.widget.SetRange(100)

  def read(self):
    "Get progress bar position"
    return self.widget.GetValue() / 100.

  def write(self, value):
    "Set progress bar position"
    # negative values pulse the bar, positive values position the bar.
    if value < 0:
      self.widget.Pulse()
    else:
      self.widget.SetValue(int(round(value * 100)))


class RadioButton(Widget):
  "wx RadioButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    if self.widget.__class__ == wx.RadioBox:
      event_type = wx.EVT_RADIOBOX
    else:
      event_type = wx.EVT_RADIOBUTTON
    self.widget.Bind(event_type, self.value_changed)

  def read(self):
    "Get index of activated button"
    return self.widget.GetSelection()

  def write(self, value):
    "Set activate button indexed by value"
    self.widget.SetSelection(value)


class Slider(Widget):
  "wx Slider widget abstractor"

  def __init__(self):
    # connect relevant signals
    self.widget.Bind(wx.EVT_LEFT_UP,
      lambda event: wx.CallAfter(self.value_changed) or event.Skip())

  def read(self):
    "Get Slider value"
    return self.widget.GetValue()

  def write(self, value):
    "Set Slider value"
    self.widget.SetValue(value)


class SpinButton(Widget):
  "wx SpinButton widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    wx.EVT_SPINCTRL(self.widget, self.widget.GetId(), self.value_changed)


  def read(self):
    "Get spinbutton value"
    return self.widget.GetValue()

  def write(self, value):
    "Set spinbutton value"
    self.widget.SetValue(value)


class StatusBar(Widget):
  "wx StatusBar widget abstractor"

  def __init__(self):
    pass

  def write(self, value):
    "Set StatusBar value (only field 1)"
    self.widget.SetStatusText(value)


class TextView(Widget):
  "wx TextView widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    self.widget.get_buffer().connect("changed", self.value_changed)

  def read(self):
    "Get text from TextView"
    textbuf = self.widget.get_buffer()
    return textbuf.get_text(textbuf.get_start_iter(), textbuf.get_end_iter())

  def write(self, value):
    "Set text into TextView"
    self.widget.get_buffer().set_text(str(value))


class ToggleButton(Widget):
  "wx ToggleButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    if self.widget.__class__ == wx.CheckBox:
      event_type = wx.EVT_CHECKBOX
    else:
      event_type = wx.EVT_TOGGLEBUTTON
    self.widget.Bind(event_type, self.value_changed)

  def read(self):
    "Get button status"
    return bool(self.widget.GetValue())

  def write(self, value):
    "Set button status"
    self.widget.SetValue(value)


class TreeView(ListTreeView):
  "wx TreeView widget abstractor"

  def __init__(self):

    # check for allowed control type
    if self.connection.control_value.get('head', None):
      raise error, "%s widget do not support header." \
        % self.widget.__class__.__name__
    key, row = \
        self.connection.control_value.get('body', None).iteritems().next()
    if type(row) == list and not len(row) == 1:
      raise error, \
          "%s widget do not allow data rows with more than one column."\
        % self.widget.__class__.__name__

    # connect relevant signals
    self.widget.Bind(wx.EVT_TREE_END_LABEL_EDIT,
      lambda event: wx.CallAfter(self.value_changed) or event.Skip())

  def append_column(self, col_num, text):
    "wx TreeCtrl has no columns support"
    pass

  def read(self):
    "Get values displayed by widget"
    # get data rows
    body = {}

    def read_node(node_id, node_path):
      """recursive depth first tree data node reader"""
      try:
        body[string.join(map(str, node_path), '.')] = \
          self.row_types[0](self.widget.GetItemText(node_id))
      except:
        if self.row_types[0] == int:
          body[string.join(map(str,node_path),'.')] = [0]
        elif self.row_types[0] == float:
          body[string.join(map(str,node_path),'.')] = [0.0]
      child_id,child = self.widget.GetFirstChild(node_id)
      child_num = 1
      while child_id:
        read_node(child_id,node_path+[child_num])
        child_id,child = self.widget.GetNextChild(node_id,child)
        child_num += 1
    root_id = self.widget.GetRootItem()
    child_id,child = self.widget.GetFirstChild(root_id)
    child_num = 1
    while child_id:
      read_node(child_id,[child_num])
      child_id,child = self.widget.GetNextChild(root_id,child)
      child_num += 1
    # return
    return {'body': body}

  def write_head(self,head):
    "Header not supported"
    pass

  def root_node(self):
    "Clean tree and return the root node"
    self.widget.DeleteAllItems()
    return self.widget.AddRoot('')

  def add_node(self,parent,last_node,current_depth,data):
    "Add current node to the tree"
    return self.widget.AppendItem(parent,str(data))

#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : A counter with count speed control (Tk)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	17-Nov-2006
# .copyright  : (c) 2006 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


from Tkinter import *			# Tk interface

from avc import *			# AVC

LOW_SPEED = 500				#--
HIGH_SPEED = 100			#- low and high speed count period (ms)


class Example(AVC):
  """
  A counter displayed in a Label widget whose count speed can be doubled
  by pressing a Toggle Button.
  """

  def __init__(self):

    # create GUI
    self.root = Tk()
    self.root.title('AVC Tk counter example')
    self.frame = Frame(self.root,name='frame')
    self.frame.pack()
    self.label = Label(self.frame,name='counter')
    self.label.pack(side=LEFT)
    self.check_button = \
        Checkbutton(self.frame,name='high_speed',text="high\nspeed")
    self.check_button.pack(side=RIGHT)

    # the counter variable and its speed status
    self.counter = 0
    self.high_speed = False

    # start counter incrementer at low speed
    self.root.after(LOW_SPEED,self.incrementer) 


  def incrementer(self):
    """
    Counter incrementer: increment period = LOW_SPEED, if high speed is False,
    increment period = HIGH_SPEED otherwise.
    """
    self.counter += 1
    if self.high_speed:
      period = HIGH_SPEED
    else:
      period = LOW_SPEED
    self.root.after(period,self.incrementer) 

  def high_speed_changed(self,value):
    "Notify change of counting speed to terminal"
    if value:
      print 'counting speed changed to high'
    else:
      print 'counting speed changed to low'


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
mainloop()			 	# run Tk event loop until quit


#### END



#!/usr/bin/env jython
# .+
# .context    : Application View Controller
# .title      : table widget display capabilities (Swing),
#		GUI programmatically generated
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Torino - Italy
# .creation   :	29-Oct-2009
# .copyright  : (c) 2009 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-

from javax import swing			# swing toolkit bindings
from java import awt			# awt toolkit bindings

from avc import *			# AVC for Swing

import copy				# object cloning support

UPDATE_PERIOD = 2000			# ms


class Example(AVC):
  """
  Showcase of display capabilities for the table widget
  """

  def __init__(self):

    # create GUI
    root = swing.JFrame('AVC Swing table example',size=(500,120),
      defaultCloseOperation = swing.JFrame.EXIT_ON_CLOSE)
    root.layout = awt.FlowLayout()
    root.add(swing.JLabel(name='list__label',))
    table = swing.JTable(name='list__table')
    scrollpane = swing.JScrollPane()
    scrollpane.setPreferredSize(awt.Dimension(200,65))
    scrollpane.getViewport().setView(table)
    root.add(scrollpane)
    root.show()

    # connected variables
    self.list = {'head':['col1 int','col2 str'], \
      'body':[[1,'one'],[2,'two'],[3,'three']]}
    self.list_work = copy.deepcopy(self.list)

    # start variables update
    update = self.update()
    self.timer = swing.Timer(UPDATE_PERIOD,None)
    self.timer.actionPerformed = lambda event: update.next()
    self.timer.start()


  def update(self):
    """
    Tabular data rows data are rolled down.
    """
    rows_num = len(self.list['body'])
    while True:
      # save last row of data
      last_row = self.list_work['body'][-1]
      # shift down one position each data row
      for i in range(1,rows_num): 
        self.list_work['body'][-i] = \
          self.list_work['body'][-1-i]
      # copy last row into first position
      self.list_work['body'][0] = last_row
      # copy working copy into connected variable
      self.list = self.list_work
      yield


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables

#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id:$
# .context    : Application View Controller
# .title      : A spin box replicated into a text label (Qt4)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Torino - Italy
# .creation   :	10-Jan-2008
# .copyright  : (c) 2008 Fabrizio Pollastri.
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


from PyQt4.QtCore import *		# Qt core
from PyQt4.QtGui import *		# Qt GUI interface
import sys				# system support

from avc import *			# AVC


class Example(QApplication,AVC):
  """
  A spin box whose value is replicated into a text label
  """

  def __init__(self):

    ## create GUI

    QApplication.__init__(self,sys.argv)

    # main window
    self.window = QWidget()
    self.window.setWindowTitle('AVC Qt4 spin box example')
    self.window.resize(280,70)

    # horizontal layout for widgets inside main window
    self.hbox = QHBoxLayout()
    self.window.setLayout(self.hbox)

    # label replicating the spin box value with formatting string
    self.label = QLabel()
    self.label.setObjectName('spin_value__label')
    self.label.setText('<b>%d</b>')
    self.hbox.addWidget(self.label)
     
    # spin box
    self.spinbox = QSpinBox()
    self.spinbox.setRange(0,100)
    self.spinbox.setObjectName('spin_value__spinbox')
    self.hbox.addWidget(self.spinbox)

    # show all widgets
    self.window.show()


    # the variable holding the spin box value
    self.spin_value = 0


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
example.exec_()				# run Qt event loop until quit

#### END



# .+
# .context    : Application View Controller
# .title      : AVC Qt4 bindings
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	7-Nov-2006
# .copyright  :	(c) 2006-2013 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with AVC.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


#### IMPORT REQUIRED MODULES

import PyQt4.Qt as qt		# Qt4 tool kit GUI bindings
from PyQt4.QtCore import Qt # Qt4 tool kit GUI attribute bindings

import string			# string operations


#### GENERAL ABSTRACTION METHODS

def toplevel_widgets():
  "Return the list of all top level widgets"
  return qt.QApplication.topLevelWidgets()

def init(core):
  "Do init specific for this widget toolkit"
  # mapping between the real widget and the wal widget
  core['avccd'].widget_map = { \
    qt.QPushButton: 	core['Button'], \
    qt.QCalendarWidget: core['Calendar'], \
    qt.QCheckBox:   	core['ToggleButton'], \
    qt.QColorDialog:    core['ColorChooser'], \
    qt.QComboBox:   	core['ComboBox'], \
    qt.QLineEdit:	    core['Entry'], \
    qt.QLabel:		    core['Label'], \
    qt.QProgressBar:	core['ProgressBar'], \
    qt.QRadioButton:	core['RadioButton'], \
    qt.QSlider:	    	core['Slider'], \
    qt.QSpinBox:	    core['SpinButton'], \
    qt.QDoubleSpinBox:	core['SpinButton'], \
    qt.QTextEdit:   	core['TextView'], \
    qt.QTreeWidget:	    core['listtreeview']}
  # get toolkit version
  core['avccd'].toolkit_version = qt.PYQT_VERSION_STR

def widget_children(widget):
  "Return the list of all children of the widget"
  return widget.children()

def widget_name(widget):
  "Return the widget name"
  return str(widget.objectName())

def timer(function,period):
  "Start a Qt timer calling back 'function' every 'period' seconds."
  timer = qt.QTimer()
  qt.QObject.connect(timer,qt.SIGNAL("timeout()"),function)
  timer.start(int(period * 1000.0))
  return timer


#### WIDGETS ABSTRACTION LAYER (widget toolkit side)

class Widget:
  "Qt4 Widget Abstraction Layer abstract class"

  def connect_delete(self,widget,delete_method):
    "Connect widget delete method to close and destroy events"
    widget.window().setAttribute(Qt.WA_DeleteOnClose)
    qt.QObject.connect(widget,qt.SIGNAL("destroyed()"),delete_method)


class ListTreeView(Widget):
  "Support to ListView and TreeView abstractors"

  def __init__(self):
    "Init operations common to ListView and TreeView"

    # clear all items and remove all columns, if any
    self.widget.clear()

    # add required columns to TreeView widget with title (header), if required.
    head = self.connection.control_value.get('head',None)
    self.widget.setColumnCount(len(self.row_types))
    if head:
      self.widget.setHeaderLabels(head)
    else:
      pass

    # connect relevant signals
    qt.QObject.connect(self.widget.model(),
      qt.SIGNAL("layoutChanged()"),self.value_changed)
    qt.QObject.connect(self.widget.model(),
      qt.SIGNAL("dataChanged()"),self.value_changed)
    qt.QObject.connect(self.widget.model(),
      qt.SIGNAL("headerDataChanged()"),self.value_changed)


  def append_column(self,col_num,text):
   "Append a column to the TreeView"
   pass


class Button(Widget):
  "Qt4 Button widget abstractor"

  def __init__(self):
    # connect relevant signals
    qt.QObject.connect(
      self.widget,qt.SIGNAL("pressed()"),self.value_changed)
    qt.QObject.connect(
      self.widget,qt.SIGNAL("released()"),self.value_changed)

  def read(self):
    "Get button status"
    return self.widget.isDown()

  def write(self,value):
    "Set button status"
    self.widget.setDown(value)


class Calendar(Widget):
  "Qt4 Calendar widget abstractor"

  def __init__(self):
    # connect relevant signals
    qt.QObject.connect(
      self.widget,qt.SIGNAL("selectionChanged()"),self.value_changed)

  def read(self):
    "Get selected date"
    date = self.widget.selectedDate()
    return (date.year(),date.month(),date.day())

  def write(self,value):
    "Set selected date"
    self.widget.setSelectedDate(qt.QDate(*value))


class ColorChooser(Widget):
  "Qt4 ColorChooser widget abstractor"

  def __init__(self):
    # persistent color object
    self.color = qt.QColor()
    # connect relevant signals
    qt.QObject.connect(
      self.widget,qt.SIGNAL("colorSelected(QColor)"),self.value_changed)

  def read(self):
    "Get selected color"
    return self.widget.selectedColor().getRgbF()

  def write(self,value):
    "Set selected color"
    self.color.setRgbF(*value)
    self.widget.setCurrentColor(self.color)


class ComboBox(Widget):
  "Qt4 ComboBox widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    qt.QObject.connect(
      self.widget,qt.SIGNAL("activated(int)"),self.value_changed)

  def read(self):
    "Get index of selected item"
    return self.widget.currentIndex()

  def write(self,value):
    "Set selected item by its index value"
    self.widget.setCurrentIndex(value)


class Entry(Widget):
  "Qt4 Entry widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    qt.QObject.connect(
      self.widget,qt.SIGNAL("returnPressed()"),self.value_changed)

  def read(self):
    "Get text from Entry"
    return self.widget.text()
    
  def write(self,value):
    "Set text into Entry"
    self.widget.setText(str(value))


class Label(Widget):
  "Qt4 Label widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get text into Label"
    return str(self.widget.text())

  def write(self,value):
    "Set text into Label"
    self.widget.setText(value)


class ListView(Widget):
  "Qt4 TreeView widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get values displayed by widget"
    # get head
    header = self.widget.headerItem()
    head = map(lambda col_num: str(header.text(col_num)),
      range(self.widget.columnCount()))
    # get data rows
    body = []
    item = self.widget.itemAt(0,0)
    while item:
      row = []
      for col_num in range(self.cols_num):
        try:
          row.append(self.row_types[col_num](item.text(col_num)))
        except:
          if self.row_types[col_num] == int:
            row.append(0)
          elif self.row_types[col_num] == float:
            row.append(0.0)
      body.append(row)
      item = self.widget.itemBelow(row)
    # return
    return {'head': head,'body': body}


  def write(self,value):
    "Set values displayed by widget"
    # set header
    if value.has_key('head'):
      self.widget.setHeaderLabels(value['head'])
    # set data rows
    self.widget.clear()
    body = value['body']
    last_item = None
    if type(body[0]) == list:
      for row in body:
        self.widget.addTopLevelItem(qt.QTreeWidgetItem(map(str,row)))
    else:
      for row in body:
        self.widget.addTopLevelItem(qt.QTreeWidgetItem(str(row)))


class ProgressBar(Widget):
  "Qt4 ProgressBar widget abstractor"

  def __init__(self):
    # set value range to 0-100
    self.widget.setMinimum(0)
    self.widget.setMaximum(100)
    pass

  def read(self):
    "Get progress bar position"
    return self.widget.value() / 100.0
    
  def write(self,value):
    "Set progress bar position"
    # negative values pulse the bar, positive values position the bar. 
    if value < 0:
      self.widget.setMinimum(0)
      self.widget.setMaximum(0)
      self.widget.setValue(0)
    else:
      self.widget.setMinimum(0)
      self.widget.setMaximum(100)
      self.widget.setValue(int(round(value * 100)))


class RadioButton(Widget):
  "Qt4 RadioButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    qt.QObject.connect(
      self.widget,qt.SIGNAL("clicked()"),self.value_changed)

  def read(self):
    "Get index of activated button"
    return self.widget.group().checkedId()

  def write(self,value):
    "Set activate button indexed by value"
    self.widget.group().buttons()[value].setChecked(True)

 
class Slider(Widget):
  "Qt4 Slider widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    qt.QObject.connect(
      self.widget,qt.SIGNAL("valueChanged(int)"),self.value_changed)

  def read(self):
    "Get Slider value"
    return self.widget.value()

  def write(self,value):
    "Set Slider value"
    self.widget.setValue(value)


class SpinButton(Widget):
  "Qt4 SpinButton widget abstractor"

  def __init__(self):
    # connect relevant signals to handlers
    # QSpinBox manages integers, while QDoubleSpinBox manages floats
    if self.widget.__class__ == qt.QSpinBox:
      SIGNAL_NAME = "valueChanged(int)"
    else:
      SIGNAL_NAME = "valueChanged(double)"
    qt.QObject.connect(
      self.widget,qt.SIGNAL(SIGNAL_NAME),self.value_changed)

  def read(self):
    "Get spinbutton value"
    return self.widget.value()

  def write(self,value):
    "Set spinbutton value"
    self.widget.setValue(value)


class StatusBar(Widget):
  "Q4 no status bar support"
  pass


class TextView(Widget):
  "Qt4 TextView/Edit widget abstractor"

  def __init__(self):
    # connect relevant signals
    qt.QObject.connect(
      self.widget,qt.SIGNAL("textChanged()"),self.value_changed)

  def read(self):
    "Get text from TextView"
    return str(self.widget.toPlainText())
    
  def write(self,value):
    "Set text into TextView"
    self.widget.setPlainText(str(value))
 

class ToggleButton(Widget):
  "Qt4 ToggleButton widget abstractor"

  def __init__(self):
    # connect relevant signals
    qt.QObject.connect(
      self.widget,qt.SIGNAL("clicked()"),self.value_changed)

  def read(self):
    "Get button status"
    return self.widget.isChecked()

  def write(self,value):
    "Set button status"
    self.widget.setChecked(value)


class TreeView(Widget):
  "Qt4 TreeView widget abstractor"

  def __init__(self):
    pass

  def read(self):
    "Get values displayed by widget"
    # get head
    header = self.widget.headerItem()
    head = map(lambda col_num: str(header.text(col_num)),
      range(self.widget.columnCount()))
    # get data row
    body = {}
    # recursive depth first tree data node reader
    def read_node(node,node_path):
      row = []
      for col_num in range(self.cols_num):
        try:
            row.append(self.row_types[col_num](node.text(col_num)))
        except:
          if self.row_types[col_num] == int:
            row.append(0)
          elif self.row_types[col_num] == float:
            row.append(0.0)
      body[string.join(map(str,node_path),'.')] = row
      for child_num in range(node.childCount()):
        read_node(node.child(child_num),node_path+[child_num+1])
    for node_num in range(self.widget.topLevelItemCount()):
      read_node(self.widget.topLevelItem(node_num),[node_num+1])
    # return
    return {'head': head,'body': body}

  def write_head(self,head):
    self.widget.setHeaderLabels(head)

  def root_node(self):
    "Return the root node of the tree"
    return self.widget

  def add_node(self,parent,last_node,current_depth,data):
    "Add current node to the tree"
    node = qt.QTreeWidgetItem(map(str,data))
    if current_depth == 1:
      self.widget.addTopLevelItem(node)
    else:
      parent.addChild(node)
    return node
 
#### END



#!/usr/bin/python
# .+
#
# .identifier :	$Id: example2.py,v 1.1 2006/11/20 17:31:56 fabrizio Exp $
# .context    : Application View Controller 
# .title      : A table of all supported widget/control type conbinations (Tk)
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	6-Jun-2007
# .copyright  : (c) 2007 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


from Tkinter import * 			# Tk interface

from avc import *			# AVC

TCL_FILE = 'tk_showcase.tcl'		# GUI description as tcl script
INCREMENTER_PERIOD = 0.333		# seconds


class Example(AVC):
  """
  A table of all supported widget/control type combinations
  """

  def __init__(self):

    # create GUI
    self.root = Tk()
    self.root.eval('set argc {}; set argv {}; proc ::main {argc argv} {};')
    self.root.tk.evalfile(TCL_FILE)

    # terminate program at toplevel window destroy: connect toplevel
    # destroy signal to termination handler.
    self.root.bind_class('Toplevel','<Destroy>',lambda event: self.root.quit())

    # the control variables
    self.boolean1 = False
    self.boolean2 = False
    self.radio = 0
    self.integer = 0
    self.float = 0.0
    self.string = ''
    self.textview = ''

    # start variables incrementer
    increment = self.incrementer()
    self.timer_function = increment.next
    self.root.after(int(INCREMENTER_PERIOD * 1000.0),self.timer_wrap)


  def timer_wrap(self):
    "Call given function, reschedule it after return"
    self.timer_function()
    self.root.after(int(INCREMENTER_PERIOD * 1000.0),self.timer_wrap)


  def incrementer(self):
    """
    Booleans are toggled, radio button index is rotated from first to last,
    integer is incremented by 1, float by 0.5, string is appended a char
    until maxlen when string is cleared, text view/edit is appended a line
    of text until maxlen when it is cleared.
    Return True to keep timer alive.
    """
    while True:
      self.boolean1 = not self.boolean1
      yield True

      self.boolean2 = not self.boolean2
      yield True

      if self.radio == 2:
        self.radio = 0
      else:
        self.radio += 1
      yield True

      self.integer += 1
      yield True

      self.float += 0.5
      yield True

      if len(self.string) >= 20:
        self.string = 'A'
      else:
        self.string += 'A'
      yield True

      if len(self.textview) >= 200:
        self.textview = ''
      else:
        self.textview += 'line of text, line of text, line of text\n'
      yield True


#### MAIN

example = Example()			# instantiate the application
example.avc_init()			# connect widgets with variables
mainloop()				# run Tk event loop until quit

#### END



# -*- coding: utf-8 -*-
# .+
# .context    : Application View Controller
# .title      : AVC core
# .kind	      : python source
# .author     : Fabrizio Pollastri
# .site	      : Revello - Italy
# .creation   :	3-Nov-2006
# .copyright  :	(c) 2006-2015 Fabrizio Pollastri
# .license    : GNU General Public License (see below)
#
# This file is part of "AVC, Application View Controller".
#
# AVC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# AVC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with AVC.  If not, see <http://www.gnu.org/licenses/>.
#
# .-


# import required modules
import copy				# object deep copy
import sys				# command line option reading


class Error(Exception):
    """A generic error exception"""

    def __init__(self, value):
        Exception.__init__(self)
        self.value = value

    def __str__(self):
        return repr(self.value)


## module information
__author__ = 'Fabrizio Pollastri <f.pollastri@inrim.it>'
__license__ = '>= GPL v3'
__version__ = '0.9.0'


## load proper AVC widget toolkit binding according with the widget toolkit
# imported by application program

# supported toolkit names indexed by python binding module names
TOOLKITS = { 'gtk':'GTK+','PyQt4':'Qt4','javax':'Swing', \
  'Tkinter':'Tkinter','wx':'wxWidgets'}

# avc toolkit bindings indexed by python binding module names
AVC_BINDINGS = { 'gtk':'avcgtk','PyQt4':'avcqt4', \
  'javax':'avcswing','Tkinter':'avctk', 'wx':'avcwx'}

AVC_PREFIX = 'avc.'

# test which widget toolkit binding module is imported: if none raise an error.
for toolkit in TOOLKITS.keys():
  if sys.modules.has_key(toolkit):
    break
else:
  raise Error,'No supported toolkit found: import it before AVC import.'

# found a supported toolkit: import its AVC binding
REAL = __import__(AVC_PREFIX + AVC_BINDINGS[toolkit],globals(),locals(),
  [AVC_BINDINGS[toolkit]])


# command line option switches
OPT_VERBOSITY = '--avc-verbosity'

# separator between widget name part 1 and 2
WIDGET_NAME_SEP = '__'

# AVC common data
class AVCcd:
  def __init__(self):
    self.toolkit_version = ''
    self.verbosity = 0
    self.view_period = 0.1
    self.widget_map = {}
    self.cogets = {}
    self.connections = {}
    self.connections_updates = {}
    self.connected_widgets = {}
    self.timer = None
avccd = AVCcd()


class AVC(object):
  "AVC programming interface"

  def avc_init(self,verbosity=0,view_period=0.1):
    "Init AVC core logic"

    # save parameters as globals
    avccd.verbosity = verbosity
    avccd.view_period = view_period

    # if any, get options from command line and override init arguments
    try:
      opt_switch_index = sys.argv.index(OPT_VERBOSITY)
      avccd.verbosity = int(sys.argv[opt_switch_index+1])
    except:
      pass

    # do init specific to widget toolkit
    real.init(globals())

    # if verbosity > 0 , print header
    if avccd.verbosity > 0:
      print 'AVC ' + __version__ + ' - Activity Report'
      print 'widget toolkit binding: ' + TOOLKITS[toolkit] + ' v' \
        + avccd.toolkit_version
      print 'program: ' + sys.argv[0]  
      print 'verbosity: ' + str(avccd.verbosity)
      if avccd.view_period:
        print 'connection update mode: periodic, period=' + \
          str(avccd.view_period) + ' sec'
      else:
        print 'connection update mode: immediate'

    # connect widgets-variables in __main__ namespace
    self.avc_connect(real.toplevel_widgets())

    # if a sampled (periodic) update of all controls views is required,
    # start a periodic call to view update function.
    if avccd.view_period != 0.0:
      avccd.timer = real.timer(self.view_update,avccd.view_period)


  def avc_connect(self,toplevel):
    """
    For each widget at or below toplevel, having a matching name with
    an attribute of object_ object, create a widget-attribute connection.
    """

    # check for avc_init execution
    if not avccd.widget_map:
      raise Error, "avc_connect must be called after avc_init call."

    # force top level widgets to be a list
    if toplevel.__class__ != list:
      toplevel = [toplevel]

    if avccd.verbosity > 3:
      print 'widget tree scansion from top level ' + str(toplevel)

    # for each widget in GUI ... 
    for widget, widget_name in self.get_widget(toplevel):

      # if widget is not supported: go to next widget
      if not avccd.widget_map.has_key(widget.__class__):

        if avccd.verbosity > 3:
	  print '  skip unsupported widget ' + \
	    widget.__class__.__name__ + ',"' + widget_name + '"'

        continue

      # if the widget is already connected: go to next widget. 
      if avccd.connected_widgets.has_key(widget):

        if avccd.verbosity > 3:
	  print '  skip already connected widget ' + \
	    widget.__class__.__name__ + ',"' + widget_name + '"'

        continue
	
      # control name is the widget name part before WIDGET_NAME_SEP string,
      # if present, otherwise is the whole widget name.
      control_name = widget_name.split(WIDGET_NAME_SEP)[0]

      # if no object attribute with the same name: go to next widget. 
      if not hasattr(self,control_name):

        if avccd.verbosity > 3:
	  print '  skip unmatched widget ' + \
	    widget.__class__.__name__ + ',"' + widget_name + '"'

        continue
	
      ## there exists an application attribute with the same name

      # if the connection exists, get it from connections dictionary,
      # if the connection does not exists, create it.
      connection = avccd.connections.setdefault((control_name,self), \
        Connection(getattr(self,control_name)))

      # add widget to connection and mark widget as connected
      connection.add_widget(control_name,self,widget,widget_name)
      avccd.connected_widgets[widget] = connection


  def get_widget(self,widgets):
    """
    Widget tree iterator. Start from toplevel widgets and traverse their
    widgets trees in breath first mode returning for each widget its
    pointer and name.
    """
    # for each toplevel widget ...
    while widgets:
      children = []
      # for each widget in this level ...
      for widget in widgets:
        # return pointer and name of widget
        yield (widget,real.widget_name(widget))
        children += real.widget_children(widget)
        # children of this level are widgets of next level
        widgets = children


  def view_update(self):
    "Periodically update views for all scheduled cogets"

    for connection in avccd.connections_updates.keys():
      setter = avccd.connections_updates[connection]
      # set the new control value in all widgets binded to this control
      # excluding the setting widget, if setter is a widget.
      for wal_widget in connection.wal_widgets:
        if wal_widget != setter:
          try:
            wal_widget.write(connection.control_value)
          # on writing error terminate
          except:
            print "error writing " + str(connection.control_value) \
                    + " to " + str(wal_widget) + ':\n', sys.exc_info()[1]
            sys.exit()

    # clear all update requests
    avccd.connections_updates = {}


class Connection:
  "Widgets-variable connection"

  def __init__(self,control_value=None):

    # set storage for control value, type, connected wal widget list,
    # value changed handler and coget
    self.control_value = control_value
    self.control_type = control_value.__class__
    self.wal_widgets = []
    self.set_handler = None
    self.object_ = None
    self.coget = None


  def add_widget(self,control_name,object_,widget,widget_name):
    "Add one widget to current connection"

    ## if it is the first widget, setup connection control data and coget

    if not self.wal_widgets:

      # save connection object instance
      self.object_ = object_

      # if exists a object method with the name control_name+'_changed',
      # store it, it will be called when a widget set a new control value.
      if hasattr(object_,control_name + '_changed'):
        self.set_handler = getattr(object_,control_name + '_changed')

      # if the corresponding coget does not exists, create it, if exists,
      # get it from cogets dictionary.
      if avccd.cogets.has_key((control_name,object_.__class__)):
        self.coget = avccd.cogets[(control_name,object_.__class__)]
      else:
        self.coget = avccd.cogets[(control_name,object_.__class__)] = \
          Coget(control_name,object_)
        # set coget as an property in place of application variable.
        setattr(object_.__class__,control_name,self.coget)
	
      # save connection reference into coget
      self.coget.add_connection(self)

      if avccd.verbosity > 0:
        print '  creating connection "' + control_name + '" in ' + str(object_)
        print '    type: ' + str(self.control_type)
        print '    initial value: ' + str(self.control_value)
        if self.set_handler:
          print '    connected handler '+'"'+control_name+'_changed"'

    # map the connected widget into the corresponding abstract widget
    wal_widget = avccd.widget_map[widget.__class__]

    if avccd.verbosity > 1:
      print '  add widget ' + widget.__class__.__name__ + \
        ',"' + widget_name + '" to connection "' + control_name + '"'

    # add it to the wal widget list
    self.wal_widgets.append(wal_widget(self,widget))

    # init it with the control value
    self.wal_widgets[-1].write(self.control_value)


  def remove_widget(self,wal_widget):
    """
    Remove one widget from current connection. If it is the last
    one in the connection, delete the connection.
    """

    self.wal_widgets.remove(wal_widget)

    if avccd.verbosity > 1:
      print 'removing widget ' + wal_widget.widget.__class__.__name__ + \
        ' from connection "' + self.coget.control_name + \
	'" of ' + str(self.object_)

    # clear wal widget data
    del wal_widget.connection
    del wal_widget.widget

    # if connection has no more widgets delete it
    if not self.wal_widgets:

      if avccd.verbosity > 0:
        print 'removing connection "' + self.coget.control_name + \
          '" from ' + str(self.object_)

      # delete connection from general connection dictionary
      del avccd.connections[(self.coget.control_name,self.object_)]

      # delete connection from coget
      self.coget.remove_connection(self)

      # delete connection from those with a pending update
      if avccd.connections_updates.has_key(self):
        del avccd.connections_updates[self]

      # clear connection data
      del self.control_value
      del self.control_type
      del self.wal_widgets
      del self.set_handler
      del self.object_
      del self.coget


class Coget(object):
  "A control object as data descriptor"

  def __init__(self,control_name,object_):
    "Create the coget control and bind it to one attribute in object"
    # save argument
    self.control_name = control_name
    self.connections = []

  def add_connection(self,connection):
    "Add a connection"
    self.connections.append(connection)

  def remove_connection(self,connection):
    "Remove a connection. If it is the last one, delete coget."
    self.connections.remove(connection)
    if not self.connections:
      del avccd.cogets[(self.control_name,connection.object_.__class__)]
      del self.control_name
      del self.connections

  def __get__(self,object_,classinfo):
    "Get control value"
    return avccd.connections[(self.control_name,object_)].control_value


  def __set__(self,object_,value,setter=None,connection=None):
    """
    Set a new control value into application control variable. If setter
    is a widget (setter != None), call the application set handler, if exists.
    Update control view in all widgets binded to the control, if setter is
    a widget, do not update it.
    """
    # if not given, get or create connection
    if not connection:
      # if the connection exists, get it from connections dictionary,
      # otherwise, create it.
      if avccd.connections.has_key((self.control_name,object_)):
        connection = avccd.connections[(self.control_name,object_)]
      else:
        connection = avccd.connections[(self.control_name,object_)] = \
          Connection(value)
        return

    # if control old value equal to the new one, return immediately.
    if value == connection.control_value:
      return

    # set new control value: if control is a mutable sequence (list) or
    # mapping (dict), a full copy inside the coget is needed to test if it
    # is really changed.
    if connection.control_type in (list,dict):
      connection.control_value = copy.deepcopy(value)
    else:
      connection.control_value = value

    # if setter is a widget, call the application set handler for this
    # control, if exists.
    if setter and connection.set_handler:
      connection.set_handler(value)

    # if a sampled view update is required, schedule this coget for
    # view update.
    if avccd.view_period != 0.0:
      avccd.connections_updates[connection] = setter
      return
      
    # if an immediate update is required, set the new control value
    # in all widgets binded to this control excluding the setting
    # widget, if setter is a widget.
    for wal_widget in connection.wal_widgets:
      if wal_widget != setter:
        wal_widget.write(value)


  def __delete__(self,instance):
    "Cogets cannot be deleted"
    raise Error,"Trying to delete "+ str(self) +": Cogets cannot be deleted."


#### WIDGETS ABSTRACTION LAYER (coget side)

class Widget:
  "Widget Abstraction Layer abstract class"

  def __init__(self,connection,widget,allowed_types=None):
  
    # check for supported control type
    if allowed_types and not connection.control_type in allowed_types:
      raise Error, "Control type '%s' not supported with '%s' widget" % \
        (connection.control_type.__name__,widget.__class__.__name__)

    # save references
    self.connection = connection
    self.widget = widget

    # connect signal common to all widgets
    self.connect_delete(widget,self.delete)


  def read(self):
    raise Error,"Method \"read\" of abstract class Widget is undefined"

  def write(self,value):
    raise Error,"Method \"write\" of abstract class Widget is undefined"

  def value_changed(self,*args):
    "widget value changed handler"
    # set new value into control variable
    Coget.__set__(
      self.connection.coget,'',self.read(),self,self.connection)
 
  def delete(self,*args):
    "delete widget from connection"
    self.connection.remove_widget(self)


class ListTreeView(Widget,real.ListTreeView):

  def __init__(self,connection,listtreeview):
    "Common init operations for ListView and TreeView abstractors"

    # generic abstract widget init
    Widget.__init__(self,connection,listtreeview,(dict,))

    # check for allowed control type
    head = connection.control_value.get('head',None)
    if head and type(head) != list:
      raise Error, "%s widget do not allow '%s' type as header, use a list." \
      % (listtreeview.__class__.__name__,type(head).__name__)

    # save column number
    self.cols_num = len(self.row_types)

    # check for header size equal to column number
    if head and len(head) != self.cols_num:
      raise Error, "%s widget require header lenght equal to data row size." \
      % listtreeview.__class__.__name__

    # real common init
    real.ListTreeView.__init__(self)

    # add required columns to TreeView widget with title (header), if required.
    head = self.connection.control_value.get('head',None)
    if head:
      map(self.append_column,range(len(self.row_types)),head)
    else:
      map(lambda col_num: self.append_column(col_num,''),
        range(len(self.row_types)))


class Button(real.Button,Widget):
  "Button widget abstractor"

  def __init__(self,connection,button):
    # generic abstract widget init
    Widget.__init__(self,connection,button,(bool,))
    # real widget init
    real.Button.__init__(self)


class Calendar(real.Calendar,Widget):
  "Calendar widget abstractor"

  def __init__(self,connection,calendar):
    # generic abstract widget init
    Widget.__init__(self,connection,calendar,(list,tuple))
    # real widget init
    real.Calendar.__init__(self)


class ComboBox(real.ComboBox,Widget):
  "ComboBox widget abstractor"

  def __init__(self,connection,combobox):
    # generic abstract widget init
    Widget.__init__(self,connection,combobox,(int,))
    # real widget init
    real.ComboBox.__init__(self)


class ColorChooser(real.ColorChooser,Widget):
  "ColorChooser widget abstractor"

  def __init__(self,connection,colorchooser):
    # generic abstract widget init
    Widget.__init__(self,connection,colorchooser,(list,tuple))
    # real widget init
    real.ColorChooser.__init__(self)


class Entry(real.Entry,Widget):
  "Entry widget abstractor"

  def __init__(self,connection,entry):
    # generic abstract widget init
    Widget.__init__(self,connection,entry,(float,int,str))
    # real widget init
    real.Entry.__init__(self)

  def read(self):
    "Get Entry value"
    return self.connection.control_type(real.Entry.read(self))


class Label(real.Label,Widget):
  "Label widget abstractor"

  def __init__(self,connection,label):

    # generic abstract widget init
    Widget.__init__(self,connection,label)

    # check for generic python object
    if connection.control_type in (bool,float,int,list,str,tuple,dict):
      self.is_object = False
      control_value = connection.control_value
    else:
      self.is_object = True
      control_value = connection.control_value.__dict__
        
    # get default format string, if any.
    self.format = str(self.read())

    # check for a working format
    try:
      if connection.control_type == list:
        junk = self.format % tuple(control_value)
      elif connection.control_type == dict:
        junk = self.format % control_value
        if junk == self.format:
          raise
      else:
        junk = self.format % control_value
      if avccd.verbosity > 2:
        print '    valid format string: "' + self.format + '"'
    except:
      if avccd.verbosity > 2:
        if self.format:
          print '    invalid format string: "' + self.format + '"'
        else:
          print '    no format string'
      self.format = None

    # real widget init
    real.Label.__init__(self)


  def read(self):
    "Get value from Label"
    # if control type is a generic object do not coerce to its type
    if self.is_object:
      return real.Label.read(self)
    # if control type not a generic object,first try to coerce to control type
    try:
      return self.connection.control_type(eval(real.Label.read(self)))
    # if fail, return value as string, needed for format string initial get.
    except:
      return real.Label.read(self)

  def write(self,value):
    "Set text into Label"
    if self.format:
      if self.is_object:
        real.Label.write(self,self.format % value.__dict__)
      else:
        if type(value) == list:
          value = tuple(value)
        real.Label.write(self,self.format % value)
    else:
      real.Label.write(self,str(value))


class ListView(real.ListView,ListTreeView):
  "ListView widget abstractor"

  def __init__(self,connection,listview):

    # save data row types
    body = connection.control_value.get('body',None)
    if type(body[0]) == list:
      self.row_types = map(type,body[0])
    else:
      self.row_types = [type(body[0])]

    # common init
    ListTreeView.__init__(self,connection,listview)

    # real widget init
    real.ListView.__init__(self)


class ProgressBar(real.ProgressBar,Widget):
  "ProgressBar widget abstractor"

  def __init__(self,connection,progressbar):
    # generic abstract widget init
    Widget.__init__(self,connection,progressbar,(float,int))
    # real widget init
    real.ProgressBar.__init__(self)

  def read(self):
    "Get Entry value"
    return self.connection.control_type(real.Entry.read(self))


class RadioButton(real.RadioButton,Widget):
  "RadioButton widget abstractor"

  def __init__(self,connection,radiobutton):
    # generic abstract widget init
    Widget.__init__(self,connection,radiobutton,(int,))
    # real widget init
    real.RadioButton.__init__(self)


class Slider(real.Slider,Widget):
  "Slider widget abstractor"

  def __init__(self,connection,slider):
    # generic abstract widget init
    Widget.__init__(self,connection,slider,(float,int))
    # real widget init
    real.Slider.__init__(self)

  def read(self):
    "Get Slider value"
    return self.connection.control_type(real.Slider.read(self))


class SpinButton(real.SpinButton,Widget):
  "SpinButton widget abstractor"

  def __init__(self,connection,spinbutton):
    # generic abstract widget init
    Widget.__init__(self,connection,spinbutton,(float,int))
    # real widget init
    real.SpinButton.__init__(self)

  def read(self):
    "Get spinbutton value"
    return self.connection.control_type(real.SpinButton.read(self))


class StatusBar(real.StatusBar,Widget):
  "StatusBar widget abstractor"

  def __init__(self,connection,statusbar):
    # generic abstract widget init
    Widget.__init__(self,connection,statusbar,(str,))
    # real widget init
    real.StatusBar.__init__(self)


class TextView(real.TextView,Widget):
  "TextView widget abstractor"

  def __init__(self,connection,textview):
    # generic abstract widget init
    Widget.__init__(self,connection,textview,(str,))
    # real widget init
    real.TextView.__init__(self)


class ToggleButton(real.ToggleButton,Widget):
  "ToggleButton widget abstractor"

  def __init__(self,connection,togglebutton):
    # generic abstract widget init
    Widget.__init__(self,connection,togglebutton,(bool,))
    # real widget init
    real.ToggleButton.__init__(self)


class TreeView(real.TreeView,ListTreeView):
  "TreeView widget abstractor"

  def __init__(self,connection,treeview):

    # save data row types
    body = connection.control_value.get('body',None)
    self.row_types = map(type,body.itervalues().next())

    # common init
    ListTreeView.__init__(self,connection,treeview)

    # real widget init
    real.TreeView.__init__(self)


  def write(self,value):
    "Set values displayed by widget"
    # set header
    if value.has_key('head'):
      self.write_head(value['head'])
    # set data rows
    body = value['body']
    # sort node ids in depth first order
    node_ids = body.keys()
    nodes = zip(map(lambda id: map(int,id.split('.')),node_ids),node_ids)
    nodes.sort()
    # depth first tree data node writer
    current_depth = 1
    parents = [self.root_node()]
    last_node = None
    for node in nodes:
      node_depth = len(node[0])
      if node_depth > current_depth:
        parents.append(last_node)
        current_depth = node_depth
      elif node_depth < current_depth:
        last_node = parents.pop(-1)
        current_depth = node_depth
      last_node = self.add_node(parents[-1],last_node,current_depth,
        body[node[1]])
 

## support functions

# support for ListView and TreeView abstractors"

def listtreeview(connection,treeview):
  """
  Route real tree view widgets supporting both list and tree data structures
  to the abstract widgets supporting only list or tree data.
  """
  body_type = type(connection.control_value.get('body',None))
  if body_type == list:
    return ListView(connection,treeview)
  elif body_type == dict:
    return TreeView(connection,treeview)
  else:
    raise Error, "%s widget do not allow '%s' type as data," \
    + "use a list for tabular data or a dictionary for tree data." \
    % (treeview.__class__.__name__,type(body).__name__)


#### END



#!/usr/bin/python

import gtk

window = gtk.Window()
window.connect('destroy',gtk.main_quit)

calendar = gtk.Calendar()
window.add(calendar)

print '1 ', calendar.get_date()
calendar.select_day(1)
print '2 ', calendar.get_date()
calendar.select_month(2,2000)
print '3 ', calendar.get_date()

window.show_all()

gtk.main()
